@@ -32,11 +32,6 @@ struct sctp_initmsg;
 struct sctp_sack_info;
 struct sctp_status;
 
-struct gensio_lock;
-struct gensio_timer;
-struct gensio_runner;
-struct gensio_waiter;
-
 /* I/O descriptor. */
 struct gensio_iod {
     /*
@@ -49,6 +44,7 @@ struct gensio_iod {
 enum gensio_iod_type {
     GESNIO_IOD_INVALID, /* Not used for anything the user will see. */
     GENSIO_IOD_SOCKET,
+    /* User cannot allocate these with add_iod().  See note in gensio_win.c */
     GENSIO_IOD_PIPE,
     GENSIO_IOD_DEV,
     GENSIO_IOD_FILE,
@@ -69,6 +65,27 @@ enum gensio_iod_type {
      */
     GENSIO_IOD_CONSOLE,
 
+    /*
+     * PTY iods create a pseudoterminal on the system.  This does not
+     * start the pty, though.  You have to call the START control to
+     * enable it.  On *nix systems, you can fetch the fd and create
+     * a symlink to the device, set permissions, or whatnot.
+     *
+     * If you do not set the ARGV via control, on *nix systems it will
+     * create a PTY that another process can connect to.  On Windows
+     * this will return an error.
+     *
+     * If you set the ARGV (and optionally ENV) before calling START,
+     * this will start a new program with the given arguments (and
+     * environoment) with the pty as stdio when you call START.  On
+     * Unix it will use the effective UID/GUI for the program.  On
+     * Windows it will use the impersonation token of the calling
+     * thread.
+     *
+     * fd is currently not used, you should pass in zero.
+     */
+    GENSIO_IOD_PTY,
+
     /* Must be last */
     NR_GENSIO_IOD_TYPES
 };
@@ -86,19 +103,6 @@ struct gensio_opensocks
     int flags;
 };
 
-#define GENSIO_LOG_MASK_ALL (1 << GENSIO_LOG_FATAL | 1 << GENSIO_LOG_ERR | \
-	1 << GENSIO_LOG_WARNING | 1 << GENSIO_LOG_INFO | 1 << GENSIO_LOG_DEBUG)
-/*
- * A bitmask of log levels to tell what to log.  Defaults to fatal and err
- * only.
- */
-GENSIO_DLL_PUBLIC
-void gensio_set_log_mask(unsigned int mask);
-GENSIO_DLL_PUBLIC
-unsigned int gensio_get_log_mask(void);
-GENSIO_DLL_PUBLIC
-const char *gensio_log_level_to_str(enum gensio_log_levels level);
-
 /*
  * Flags for opensock_flags.  For the set function, add the _SET_
  * flags for the options you want to set, and set the option bits for
@@ -171,6 +175,13 @@ const char *gensio_log_level_to_str(enum gensio_log_levels level);
 /* Return the multicast loop value, see set value. */
 #define GENSIO_SOCKCTL_GET_MCAST_LOOP	9
 
+/*
+ * For UDP sockets, return the destination address and interface for
+ * received packets in the recvfrom call.
+ */
+#define GENSIO_SOCKCTL_SET_EXTRAINFO	10
+#define GENSIO_SOCKCTL_GET_EXTRAINFO	11
+
 /******************************************************************
  * For iod_control()
  */
@@ -252,6 +263,23 @@ const char *gensio_log_level_to_str(enum gensio_log_levels level);
 /* Windows sockets only, is the socket closed? */
 #define GENSIO_IOD_CONTROL_IS_CLOSED 26
 
+/* For ptys, set the window size. */
+#define GENSIO_IOD_CONTROL_WIN_SIZE 27
+/*
+ * Used to inform a pty of a new window size.  Based on the *nix
+ * winsize type, but used for windows, too.  For windows the pixel
+ * values are ignored.
+ */
+struct gensio_winsize {
+    int ws_row;
+    int ws_col;
+    int ws_xpixel;
+    int ws_ypixel;
+};
+
+/* For ptys, will cd to this directory at startup. */
+#define GENSIO_IOD_CONTROL_START_DIR 28
+
 /*
  * These are for communication between the socket code and the iod, so
  * the socket code can store information in the IOD.  It's only for
@@ -260,14 +288,17 @@ const char *gensio_log_level_to_str(enum gensio_log_levels level);
 #define GENSIO_IOD_CONTROL_SOCKINFO	1000
 
 /*
- * Used by wait functions and general handling for process setup and
- * cleanup.
+ * Operations for PTYs.  See the discussion baove GENSIO_IOD_PTY for
+ * details.
  */
-struct gensio_os_proc_data;
+#define GENSIO_IOD_CONTROL_ARGV		2000
+#define GENSIO_IOD_CONTROL_ENV		2001
+#define GENSIO_IOD_CONTROL_START	2002
 
-/*
- * Operations for the control function in gensio_os_funcs
- */
+/* Get pid, val is a ptr to intptr_t. */
+#define GENSIO_IOD_CONTROL_PID		2003
+/* Stop the process, but do not close the child process handle. */
+#define GENSIO_IOD_CONTROL_STOP		2004
 
 /*
  * Set the proc data for the os handler.  The struct
@@ -284,7 +315,7 @@ struct gensio_os_funcs {
 
     /****** Memory Allocation ******/
     /* Return allocated and zeroed data.  Return NULL on error. */
-    void *(*zalloc)(struct gensio_os_funcs *f, unsigned int size);
+    void *(*zalloc)(struct gensio_os_funcs *f, gensiods size);
 
     /* Free data allocated by zalloc. */
     void (*free)(struct gensio_os_funcs *f, void *data);
@@ -625,21 +656,25 @@ struct gensio_os_funcs {
      * to stdout and rstderr should be NULL.  Otherwise if you want to
      * leave stderr to the caller's stderr, then set rstderr to NULL.
      *
+     * If pty is set, then stderr should be passed in as NULL.
+     *
      * Note that you must wait for the subprogram after you think it
      * has terminated, and the wait must return without error, to
      * avoid leaking resources.
      */
+#define GENSIO_EXEC_STDERR_TO_STDOUT	(1 << 0)
     int (*exec_subprog)(struct gensio_os_funcs *o,
 			const char *argv[], const char **env,
-			bool stderr_to_stdout,
+			const char *start_dir,
+			unsigned int flags,
 			intptr_t *rpid,
 			struct gensio_iod **rstdin,
 			struct gensio_iod **rstdout,
 			struct gensio_iod **rstderr);
     /*
-     * Attempt to stop a subprogram.  If force is given, don't get the
-     * subprogram an option (kill -1).  You still must wait on the
-     * subprogramm.
+     * Attempt to stop a subprogram.  If force is given, don't give the
+     * subprogram an option (kill -9).  You still must wait on the
+     * subprogram.
      */
     int (*kill_subprog)(struct gensio_os_funcs *o, intptr_t pid, bool force);
     /* Wait for a program to terminate. */
@@ -649,27 +684,6 @@ struct gensio_os_funcs {
     int (*addr_create)(struct gensio_os_funcs *o,
 		       int nettype, const void *iaddr, gensiods len,
 		       unsigned int port, struct gensio_addr **newaddr);
-    bool (*addr_equal)(const struct gensio_addr *a1,
-		       const struct gensio_addr *a2,
-		       bool compare_ports, bool compare_all);
-    int (*addr_to_str)(const struct gensio_addr *addr,
-		       char *buf, gensiods *pos, gensiods buflen);
-    int (*addr_to_str_all)(const struct gensio_addr *addr,
-			   char *buf, gensiods *pos, gensiods buflen);
-    struct gensio_addr *(*addr_dup)(const struct gensio_addr *iaddr);
-    struct gensio_addr *(*addr_cat)(const struct gensio_addr *addr1,
-				    const struct gensio_addr *addr2);
-    bool (*addr_addr_present)(const struct gensio_addr *gai,
-			      const void *addr, gensiods addrlen,
-			      bool compare_ports);
-    void (*addr_free)(struct gensio_addr *addr);
-    bool (*addr_next)(struct gensio_addr *addr);
-    void (*addr_rewind)(struct gensio_addr *addr);
-    int (*addr_get_nettype)(const struct gensio_addr *addr);
-    bool (*addr_family_supports)(const struct gensio_addr *addr,
-				 int family, int flags);
-    void (*addr_getaddr)(const struct gensio_addr *addr,
-			 void *oaddr, gensiods *rlen);
 
     /*
      * Scan the str for IP addresses and create an address structure
@@ -808,68 +822,6 @@ struct gensio_os_funcs {
 		   gensiods *datalen);
 };
 
-GENSIO_DLL_PUBLIC
-void gensio_vlog(struct gensio_os_funcs *o, enum gensio_log_levels level,
-		 const char *str, va_list args);
-GENSIO_DLL_PUBLIC
-void gensio_log(struct gensio_os_funcs *o, enum gensio_log_levels level,
-		const char *str, ...);
-
-/*
- * Allocate the OS handler for the platform.  This will return the
- * same OS handler each time.  Can return GE_NOMEM if out of memory.
- */
-GENSIO_DLL_PUBLIC
-int gensio_default_os_hnd(int wake_sig, struct gensio_os_funcs **o);
-
-/* For testing, do not use in normal code. */
-GENSIO_DLL_PUBLIC
-void gensio_osfunc_exit(int rv);
-
-/*
- * Process setup for gensio OS handlers.  These are machine-specific.
- * You should call this after allocating the OS handlers and if you use
- * wait_intr_sigmask you should pass the process data from this into that
- * function.
- *
- * For Windows this currently just returns NULL data and doesn't do
- * anything.
- *
- * For Unix, this blocks SIGPIPE, SIGCHLD, and the wake signal passed
- * in to the allocation function (if the wake signal is non-zero).  It
- * then sets a sigmask to be installed on the wait_intr_sigmask with
- * the wake signal and SIGCHLD not blocked.
- *
- * It also installs signal handlers for SIGCHLD and (if non-zero) the
- * wake signal.
- *
- * For Unix this is generally what you want, you don't want SIGPIPE
- * doing bad things and having SIGCHLD wake up a wait can speed things
- * up a bit when waiting for subprograms.
- *
- * If you need to modify that signal mask used in wait_intr_signmask,
- * use gensio_os_proc_unix_get_wait_sigset() defined below to fetch it
- * and modify it.
- *
- * Note that you can override SIGCHLD and SIGPIPE if you like.  Don't
- * mess with the wake signal.
- */
-GENSIO_DLL_PUBLIC
-int gensio_os_proc_setup(struct gensio_os_funcs *o,
-			 struct gensio_os_proc_data **data);
-
-/*
- * Undo the proc setup.
- *
- * On Windows this currently does nothing.
- *
- * On Unix this restores the signal mask to what it was when
- * proc_setup was called and it removes the signal handlers it
- * installed.
- */
-GENSIO_DLL_PUBLIC
-void gensio_os_proc_cleanup(struct gensio_os_proc_data *data);
-
 /*
  * Called from os handlers, check for any handlers that may need to be
  * called.
@@ -877,61 +829,12 @@ void gensio_os_proc_cleanup(struct gensio_os_proc_data *data);
 GENSIO_DLL_PUBLIC
 void gensio_os_proc_check_handlers(struct gensio_os_proc_data *data);
 
-#ifdef _WIN32
-#define GENSIO_DEF_WAKE_SIG 0
-#else
+#ifndef _WIN32
 #include <signal.h>
-#define GENSIO_DEF_WAKE_SIG SIGUSR1
 GENSIO_DLL_PUBLIC
 sigset_t *gensio_os_proc_unix_get_wait_sigset(struct gensio_os_proc_data *data);
 #endif
 
-/*
- * Set the function to call when a termination (SIGINT, SIGQUIT,
- * SIGTERM on Unix, console control handler or WM_CLOSE on windows) is
- * requested by the operating system.  data should point to a struct
- * gensio_control_register_handler.  Set to handler NULL to disable.
- */
-GENSIO_DLL_PUBLIC
-int gensio_os_proc_register_term_handler(struct gensio_os_proc_data *data,
-					 void (*handler)(void *handler_data),
-					 void *handler_data);
-/*
- * Set the function to call when a reaload is requested by the
- * operating system (SIGHUP on Unix).  data should point to a struct
- * gensio_control_register_handler.  Set handler to NULL to disable.
- */
-GENSIO_DLL_PUBLIC
-int gensio_os_proc_register_reload_handler(struct gensio_os_proc_data *data,
-					   void (*handler)(void *handler_data),
-					   void *handler_data);
-
-
-/*
- * Basic thread handling.  You can use the standard OS functions to do
- * this, too, this is here for genericity.  It may not cover all your
- * needs.
- */
-struct gensio_thread;
-
-/*
- * Start a new thread running at start_func, passing in the given
- * data.  The thread_id is returned, use that to wait for the thread
- * to complete after it should stop.
- */
-GENSIO_DLL_PUBLIC
-int gensio_os_new_thread(struct gensio_os_funcs *o,
-			 void (*start_func)(void *data), void *data,
-			 struct gensio_thread **thread_id);
-
-/*
- * Wait for the given thread to stop.  Note that this does not cause
- * the thread to stop, it waits for the thread to stop after it has
- * been stopped to avoid race condition.
- */
-GENSIO_DLL_PUBLIC
-int gensio_os_wait_thread(struct gensio_thread *thread_id);
-
 #ifdef __cplusplus
 }
 #endif
