@@ -10,6 +10,7 @@
 #include "config.h"
 #include <assert.h>
 #include <gensio/gensio.h>
+#include <gensio/gensio_os_funcs.h>
 #include <gensio/gensio_class.h>
 #include <gensio/gensio_base.h>
 #include <gensio/gensio_builtins.h>
@@ -40,7 +41,6 @@ struct conaccn_data {
     void *close_data;
 
     unsigned int refcount;
-    unsigned int freeref;
 };
 
 enum conaccna_state {
@@ -553,12 +553,6 @@ static void
 conaccn_free(struct conaccn_data *ndata)
 {
     conaccn_lock(ndata);
-    assert(ndata->freeref > 0);
-    if (--ndata->freeref > 0) {
-	conaccn_unlock(ndata);
-	return;
-    }
-
     switch (ndata->child_state) {
     case CONACCN_IN_OPEN:
     case CONACCN_OPEN:
@@ -578,14 +572,6 @@ conaccn_free(struct conaccn_data *ndata)
     conaccn_deref_and_unlock(ndata);
 }
 
-static void
-conaccn_func_ref(struct conaccn_data *ndata)
-{
-    conaccn_lock(ndata);
-    ndata->freeref++;
-    conaccn_unlock(ndata);
-}
-
 static void
 conaccn_disable(struct conaccn_data *ndata)
 {
@@ -627,10 +613,6 @@ conaccn_func(struct gensio *io, int func, gensiods *count,
 	conaccn_free(ndata);
 	return 0;
 
-    case GENSIO_FUNC_REF:
-	conaccn_func_ref(ndata);
-	return 0;
-
     case GENSIO_FUNC_DISABLE:
 	conaccn_disable(ndata);
 	return 0;
@@ -766,7 +748,6 @@ conacc_start(struct conaccna_data *nadata)
 	goto out_err_nofree;
     ndata->o = nadata->o;
     ndata->nadata = nadata;
-    ndata->freeref = 1;
     ndata->refcount = 1;
     ndata->lock = nadata->o->alloc_lock(nadata->o);
     if (!ndata->lock)
