@@ -26,6 +26,7 @@
 #endif
 
 #include <gensio/gensio.h>
+#include <gensio/gensio_os_funcs.h>
 #include <gensio/gensio_class.h>
 #include <gensio/gensio_ll_fd.h>
 #include <gensio/argvutils.h>
@@ -48,6 +49,7 @@ struct net_data {
 
     int last_err;
 
+    bool do_oob;
     int oob_char;
 };
 
@@ -229,6 +231,8 @@ net_control(void *handler_data, struct gensio_iod *iod, bool get,
 	return 0;
 
     case GENSIO_CONTROL_LPORT:
+	if (!get)
+	    return GE_NOTSUP;
 	size = sizeof(unsigned int);
 	rv = tdata->o->sock_control(iod, GENSIO_SOCKCTL_GET_PORT, &i, &size);
 	if (rv)
@@ -236,6 +240,13 @@ net_control(void *handler_data, struct gensio_iod *iod, bool get,
 	*datalen = snprintf(data, *datalen, "%d", i);
 	return 0;
 
+    case GENSIO_CONTROL_ENABLE_OOB:
+	if (get)
+	    *datalen = snprintf(data, *datalen, "%u", tdata->do_oob);
+	else
+	    tdata->do_oob = !!strtoul(data, NULL, 0);
+	return 0;
+
     default:
 	return GE_NOTSUP;
     }
@@ -249,15 +260,18 @@ net_except_read(struct gensio_iod *iod, void *data, gensiods count,
     static const char *argv[3] = { "oob", "oobtcp", NULL };
 
     if (tdata->oob_char >= 0) {
-	*auxdata = argv;
-	if (count == 0) {
-	    *rcount = 0;
+	if (tdata->do_oob) {
+	    *auxdata = argv;
+	    if (count == 0) {
+		*rcount = 0;
+		return 0;
+	    }
+	    *((unsigned char *) data) = tdata->oob_char;
+	    tdata->oob_char = -1;
+	    *rcount = 1;
 	    return 0;
 	}
-	*((unsigned char *) data) = tdata->oob_char;
 	tdata->oob_char = -1;
-	*rcount = 1;
-	return 0;
     }
 
     return tdata->o->recv(iod, data, count, rcount, 0);
@@ -653,7 +667,7 @@ netna_readhandler(struct gensio_iod *iod, void *cbdata)
     tdata->istcp = nadata->istcp;
     tdata->nodelay = nadata->nodelay;
     raddr = NULL;
-    
+
     if (tdata->istcp)
 	setup |= GENSIO_OPENSOCK_KEEPALIVE;
     if (tdata->nodelay)
@@ -850,8 +864,10 @@ static void
 netna_cb_en_done(struct gensio_runner *runner, void *cb_data)
 {
     struct netna_data *nadata = cb_data;
+    gensio_acc_done done = nadata->cb_en_done;
 
-    nadata->cb_en_done(nadata->acc, NULL);
+    nadata->cb_en_done = NULL;
+    done(nadata->acc, NULL);
 }
 
 static int
@@ -862,6 +878,9 @@ netna_set_accept_callback_enable(struct gensio_accepter *accepter,
 {
     unsigned int i;
 
+    if (nadata->cb_en_done)
+	return GE_INUSE;
+
     nadata->cb_en_done = done;
     for (i = 0; i < nadata->nr_acceptfds; i++)
 	nadata->o->set_read_handler(nadata->acceptfds[i].iod, enabled);
