@@ -5,6 +5,11 @@
  *  SPDX-License-Identifier: LGPL-2.1-only
  */
 
+#ifdef linux
+#define _XOPEN_SOURCE 600 /* Get posix_openpt() and friends. */
+#define _GNU_SOURCE /* Get ptsname_r(). */
+#endif
+
 #include "config.h"
 #define _DEFAULT_SOURCE /* Get getgrouplist(), setgroups() */
 #include <stdio.h>
@@ -35,6 +40,7 @@ bool gensio_set_progname(const char *iprogname)
 
 #ifdef _WIN32
 #include <winsock2.h> /* For AF_UNSPEC */
+#include <windows.h>
 #include <iphlpapi.h>
 #else
 #include <sys/socket.h>
@@ -300,6 +306,7 @@ gensio_os_free_net_ifs(struct gensio_os_funcs *o,
     gensio_os_funcs_zfree(o, ifs);
 }
 
+#ifndef _WIN32
 static bool
 is_inet_family(struct sockaddr *s)
 {
@@ -307,6 +314,7 @@ is_inet_family(struct sockaddr *s)
 	return false;
     return (s->sa_family == AF_INET || s->sa_family == AF_INET6);
 }
+#endif
 
 int
 gensio_os_get_net_ifs(struct gensio_os_funcs *o,
@@ -315,7 +323,7 @@ gensio_os_get_net_ifs(struct gensio_os_funcs *o,
     struct gensio_net_if **ifs = NULL;
     char buf[100], *addrtype;
 #ifdef _WIN32
-    IP_ADAPTER_ADDRESSES *t, *c, *p;
+    IP_ADAPTER_ADDRESSES *t, *c;
     ULONG err;
     unsigned int i, j, nifs;
     ULONG buflen = 15 * 1024;
@@ -598,6 +606,11 @@ gensio_os_check_tcpd_ok(struct gensio_iod *iod, const char *iprogname)
 
 #ifdef _WIN32
 
+#include <wtsapi32.h>
+#include <ntsecapi.h>
+#include <userenv.h>
+#include <sddl.h>
+
 struct stdio_mode {
     DWORD old_mode_flags;
 };
@@ -1174,7 +1187,8 @@ win_env_to_block(struct gensio_os_funcs *o,
 int
 gensio_win_do_exec(struct gensio_os_funcs *o,
 		   const char *argv[], const char **env,
-		   bool stderr_to_stdout,
+		   const char *start_dir,
+		   unsigned int flags,
 		   HANDLE *phandle,
 		   HANDLE *rin, HANDLE *rout, HANDLE *rerr)
 {
@@ -1187,7 +1201,7 @@ gensio_win_do_exec(struct gensio_os_funcs *o,
     HANDLE stdout_m = NULL, stdout_s = NULL;
     HANDLE stderr_m = NULL, stderr_s = NULL;
 
-    if (rerr && stderr_to_stdout)
+    if (rerr && (flags & GENSIO_EXEC_STDERR_TO_STDOUT))
 	return GE_INVAL;
 
     rv = argv_to_win_cmdline(o, argv, &cmdline);
@@ -1220,7 +1234,7 @@ gensio_win_do_exec(struct gensio_os_funcs *o,
     if (!SetHandleInformation(stdout_m, HANDLE_FLAG_INHERIT, 0))
 	goto out_err_conv;
 
-    if (stderr_to_stdout) {
+    if (flags & GENSIO_EXEC_STDERR_TO_STDOUT) {
 	if (!DuplicateHandle(GetCurrentProcess(),
 			     stdout_s,
 			     GetCurrentProcess(),
@@ -1240,6 +1254,8 @@ gensio_win_do_exec(struct gensio_os_funcs *o,
 			     0, TRUE, DUPLICATE_SAME_ACCESS))
 	    goto out_err_conv;
     }
+    if (!SetHandleInformation(stderr_s, HANDLE_FLAG_INHERIT, 1))
+	goto out_err_conv;
 
     suinfo.cb = sizeof(STARTUPINFO);
     suinfo.hStdInput = stdin_s;
@@ -1251,10 +1267,10 @@ gensio_win_do_exec(struct gensio_os_funcs *o,
 		       cmdline,
 		       NULL,
 		       NULL,
-		       TRUE,
-		       0,
+		       FALSE,
+		       NORMAL_PRIORITY_CLASS,
 		       envb,
-		       NULL,
+		       start_dir,
 		       &suinfo,
 		       &procinfo))
 	goto out_err_conv;
@@ -1298,6 +1314,842 @@ gensio_win_do_exec(struct gensio_os_funcs *o,
     return rv;
 }
 
+static DWORD
+read_token_info(HANDLE h, TOKEN_INFORMATION_CLASS type, void **rval,
+		DWORD *rlen)
+{
+    DWORD err, len = 0;
+    void *val;
+
+    if (GetTokenInformation(h, type, NULL, 0, &len))
+	/* This should fail. */
+	return ERROR_INVALID_DATA;
+    err = GetLastError();
+    if (err != ERROR_INSUFFICIENT_BUFFER)
+	return err;
+    val = malloc(len);
+    if (!val)
+	return STATUS_NO_MEMORY;
+    if (!GetTokenInformation(h, type, val, len, &len)) {
+	free(val);
+	return GetLastError();
+    }
+    *rval = val;
+    if (rlen)
+	*rlen = len;
+    return 0;
+}
+
+static DWORD
+get_logon_sid_from_token(HANDLE h, SID **logon_sid, bool *found)
+{
+    DWORD err;
+    unsigned int i;
+    TOKEN_GROUPS *grps = NULL;
+    SID *sid;
+
+    err = read_token_info(h, TokenGroups, (void **) &grps, NULL);
+    if (err) {
+	/* Not an error if the token doesn't have groups. */
+	err = 0;
+	goto out_err;
+    }
+
+    /* Now scan for a logon id SID. */
+    for (i = 0; i < grps->GroupCount; i++) {
+	if ((grps->Groups[i].Attributes & SE_GROUP_LOGON_ID) ==
+	    SE_GROUP_LOGON_ID) {
+	    int len = GetLengthSid(grps->Groups[i].Sid);
+
+	    sid = (SID *) malloc(len);
+	    if (!sid) {
+		err = STATUS_NO_MEMORY;
+		goto out_err;
+	    }
+	    if (!CopySid(len, sid, grps->Groups[i].Sid)) {
+		err = GetLastError();
+		goto out_err;
+	    }
+	    *logon_sid = sid;
+	    sid = NULL;
+	    *found = true;
+	    break;
+	}
+    }
+
+ out_err:
+    if (grps)
+	free(grps);
+    if (sid)
+	free(sid);
+
+    return err;
+}
+
+static DWORD
+get_logon_sid(SID *user, SID **logon_sid, bool *rfound)
+{
+    DWORD err;
+    unsigned int i;
+    bool found = false;
+    WTS_SESSION_INFOA *sesinfo = NULL;
+    DWORD sescount;
+    TOKEN_USER *usid = NULL;
+    HANDLE h = NULL;
+
+    /*
+     * Try the calling user's token first.
+     */
+    OpenProcessToken(GetCurrentProcess(), TOKEN_QUERY, &h);
+    err = get_logon_sid_from_token(h, logon_sid, &found);
+    CloseHandle(h);
+    if (!err && found) {
+	*rfound = true;
+	return 0;
+    }
+    /*
+     * Find a session with the same user SID as the passed in user.
+     */
+    if (!WTSEnumerateSessionsA(WTS_CURRENT_SERVER_HANDLE, 0, 1,
+			       &sesinfo, &sescount)) {
+	err = GetLastError();
+	goto out_err;
+    }
+
+    for (i = 0; !found && i < sescount; i++) {
+	if (!WTSQueryUserToken(sesinfo[i].SessionId, &h))
+	    continue;
+
+	/* Check to make sure it's our user. */
+	err = read_token_info(h, TokenUser, (void **) &usid, NULL);
+	if (err)
+	    goto continue_scan;
+	if (!EqualSid(usid->User.Sid, user))
+	    goto continue_scan;
+
+	err = get_logon_sid_from_token(h, logon_sid, &found);
+	if (err)
+	    goto out_err;
+
+    continue_scan:
+	if (h) {
+	    CloseHandle(h);
+	    h = NULL;
+	}
+	if (usid) {
+	    free(usid);
+	    usid = NULL;
+	}
+    }
+
+    /* Not finding is not an error, we just report it. */
+    *rfound = found;
+    err = 0;
+
+ out_err:
+    if (h)
+	CloseHandle(h);
+    if (sesinfo)
+	WTSFreeMemory(sesinfo);
+    if (usid)
+	free(usid);
+    return err;
+}
+
+static void
+set_lsa_string(LSA_STRING *a, const char *b)
+{
+    a->Length = (USHORT) strlen(b);
+    a->MaximumLength = (USHORT)(a->Length + sizeof(*b));
+    a->Buffer = (char *) b;
+}
+
+static void
+add_unicode_str(UNICODE_STRING *ustr, const char *str, unsigned int len,
+		char **pos)
+{
+    unsigned int blen = len * sizeof(wchar_t);
+
+    ustr->Buffer = (wchar_t *) *pos;
+    mbstowcs(ustr->Buffer, str, len);
+    ustr->Length = blen;
+    ustr->MaximumLength = blen;
+    *pos += blen;
+}
+
+static DWORD
+get_kerb_logon(const char *user, const char *password, bool interactive,
+	       void **logon_info, DWORD *logon_info_len,
+	       SECURITY_LOGON_TYPE *logon_type)
+{
+    DWORD user_chars = mbstowcs(NULL, user, strlen(user));
+    DWORD user_bytes = user_chars * sizeof(wchar_t);
+    DWORD domain_chars = strchr(user, '\\') - user;
+    DWORD real_user_chars = strlen(user + domain_chars + 1);
+    DWORD logon_len;
+    char *pos;
+
+    if (password) {
+	DWORD pw_chars = mbstowcs(NULL, password, strlen(password));
+	DWORD pw_bytes = pw_chars * sizeof(wchar_t);
+	KERB_INTERACTIVE_LOGON *int_logon;
+
+	/*
+	 * KERB_S4U_LOGON must be passed as a single contiguous buffer
+	 * that includes all strings, otherwise LsaLogonUser will
+	 * complain.  Add an extra char for the termination, just in
+	 * case.
+	 */
+	logon_len = sizeof(KERB_INTERACTIVE_LOGON) + user_bytes + pw_bytes;
+	int_logon = calloc(logon_len + sizeof(wchar_t), 1);
+	if (!int_logon)
+	    return STATUS_NO_MEMORY;
+	int_logon->MessageType = KerbInteractiveLogon;
+	pos = (char *) (int_logon + 1);
+	add_unicode_str(&int_logon->LogonDomainName, user, domain_chars, &pos);
+	user += domain_chars + 1; /* SKip over domain\ */
+	add_unicode_str(&int_logon->UserName, user, real_user_chars, &pos);
+	add_unicode_str(&int_logon->Password, password, pw_chars, &pos);
+	*logon_info = int_logon;
+	*logon_type = Interactive;
+    } else {
+	/* look up the Kerb authentication provider's index */
+	KERB_S4U_LOGON *s4u_logon;
+	char *tmpstr, *upnstr;
+
+	upnstr = malloc(user_chars + 1);
+	if (!upnstr)
+	    return STATUS_NO_MEMORY;
+	tmpstr = upnstr;
+	memcpy(tmpstr, user + domain_chars + 1, real_user_chars);
+	tmpstr += real_user_chars;
+	*tmpstr++ = '@';
+	memcpy(tmpstr, user, domain_chars);
+	tmpstr += domain_chars;
+	*tmpstr = '\0';
+
+	/*
+	 * KERB_S4U_LOGON must be passed as a single contiguous buffer
+	 * that includes all strings, otherwise LsaLogonUser will
+	 * complain.  Add an extra char for the termination, just in
+	 * case.
+	 */
+	logon_len = sizeof(KERB_S4U_LOGON) + user_bytes;
+	s4u_logon = calloc(logon_len + sizeof(wchar_t), 1);
+	if (!s4u_logon) {
+	    free(upnstr);
+	    return STATUS_NO_MEMORY;
+	}
+	s4u_logon->MessageType = KerbS4ULogon;
+	if (interactive)
+	    s4u_logon->Flags = KERB_S4U_LOGON_FLAG_IDENTIFY;
+	pos = (char *) (s4u_logon + 1);
+	add_unicode_str(&s4u_logon->ClientUpn, upnstr, user_chars, &pos);
+	free(upnstr);
+	*logon_info = s4u_logon;
+	*logon_type = Network;
+    }
+    *logon_info_len = logon_len;
+    return 0;
+}
+
+static DWORD
+get_local_logon(const char *user, const char *password,
+		void **logon_info, DWORD *logon_info_len,
+		SECURITY_LOGON_TYPE *logon_type)
+{
+    DWORD user_chars = mbstowcs(NULL, user, strlen(user));
+    DWORD user_bytes = user_chars * sizeof(wchar_t);
+    DWORD logon_len;
+    char *pos;
+
+    if (password) {
+	DWORD pw_chars = mbstowcs(NULL, password, strlen(password));
+	DWORD pw_bytes = pw_chars * sizeof(wchar_t);
+	MSV1_0_INTERACTIVE_LOGON *mi_logon;
+
+	/*
+	 * MSV1_0_INTERACTIVE_LOGON must be passed as a single
+	 * contiguous buffer that includes all strings, otherwise
+	 * LsaLogonUser will complain.  Add an extra char for the
+	 * termination, just in case.
+	 */
+	logon_len = (sizeof(*mi_logon) + user_bytes + pw_bytes +
+		     sizeof(wchar_t));
+	mi_logon = (MSV1_0_INTERACTIVE_LOGON *) malloc(logon_len);
+	if (!mi_logon)
+	    return STATUS_NO_MEMORY;
+	mi_logon->MessageType = MsV1_0InteractiveLogon;
+
+	pos = (char *) (mi_logon + 1);
+	add_unicode_str(&mi_logon->LogonDomainName, ".", 1, &pos);
+	add_unicode_str(&mi_logon->UserName, user, user_chars, &pos);
+	add_unicode_str(&mi_logon->Password, password, pw_chars, &pos);
+
+	*logon_info = mi_logon;
+	*logon_type = Interactive;
+    } else {
+	MSV1_0_S4U_LOGON *s4u_logon;
+
+	/*
+	 * MSV1_0_S4U_LOGON must be passed as a single contiguous
+	 * buffer that includes all strings, otherwise LsaLogonUser
+	 * will complain.  Add an extra char for the termination, just
+	 * in case.
+	 */
+	logon_len = sizeof(MSV1_0_S4U_LOGON) + user_bytes + sizeof(wchar_t);
+	s4u_logon = (MSV1_0_S4U_LOGON *) calloc(logon_len +
+						sizeof(wchar_t), 1);
+	if (!s4u_logon)
+	    return STATUS_NO_MEMORY;
+	s4u_logon->MessageType = MsV1_0S4ULogon;
+	pos = (char *) (s4u_logon + 1);
+
+	add_unicode_str(&s4u_logon->UserPrincipalName, user, user_chars, &pos);
+	add_unicode_str(&s4u_logon->DomainName, ".", 1, &pos);
+
+	*logon_info = s4u_logon;
+	*logon_type = Network;
+    }
+    *logon_info_len = logon_len;
+    return 0;
+}
+
+static const char *add_groups[] = {
+    "S-1-2-0", /* LOCAL */
+    NULL
+};
+
+/* Supply either isid or sidstr, not both. */
+static DWORD
+append_group(TOKEN_GROUPS *grps, SID *sid, const char *sidstr, DWORD attrs)
+{
+    SID *new_sid, *free_sid = NULL;
+    size_t len;
+    unsigned int i = grps->GroupCount;
+    int err = 0;
+
+    if (sidstr) {
+	if (!ConvertStringSidToSid(sidstr, (void **) &free_sid))
+	    return GetLastError();
+	/* Can't use free_sid directly, it's allocated with LocalAlloc(). */
+	sid = free_sid;
+    }
+
+    len = GetLengthSid(sid);
+    new_sid = malloc(len);
+    if (!new_sid) {
+	err = STATUS_NO_MEMORY;
+	goto out_err;
+    }
+    if (!CopySid(len, new_sid, sid)) {
+	err = GetLastError();
+	free(new_sid);
+	goto out_err;
+    }
+
+    grps->Groups[i].Attributes = attrs;
+    grps->Groups[i].Sid = new_sid;
+    grps->GroupCount++;
+ out_err:
+    if (free_sid)
+	LocalFree(free_sid);
+    return err;
+}
+
+static void
+free_groups(TOKEN_GROUPS *grps)
+{
+    unsigned int i;
+
+    for (i = 0; i < grps->GroupCount; i++)
+	free(grps->Groups[i].Sid);
+    free(grps);
+}
+
+#if 0
+static void
+pr_sid(int indent, const char *str, SID *sid)
+{
+    char *sidstr;
+
+    if (!sid) {
+	printf("%*s%s: NULL Sid\n", indent, "", str);
+    } else if (ConvertSidToStringSidA(sid, &sidstr)) {
+	printf("%*s%s: %s\n", indent, "", str, sidstr);
+	LocalFree(sidstr);
+    } else {
+	printf("%*s%s: Bad Sid\n", indent, "", str);
+    }
+}
+
+static void
+print_sid(const char *str, SID *sid)
+{
+    pr_sid(0, str, sid);
+}
+#endif
+
+int
+gensio_win_get_user_token(const char *user, const char *password,
+			  const char *src_module, const char **groups,
+			  bool interactive, HANDLE *userh)
+{
+    HANDLE lsah;
+    NTSTATUS rv;
+    DWORD err = 0;
+    void *logon_info = NULL;
+    LSA_STRING package_name;
+    ULONG package_auth;
+    DWORD user_chars = mbstowcs(NULL, user, strlen(user));
+    DWORD user_bytes = user_chars * sizeof(wchar_t);
+    wchar_t *wuser = NULL;
+    bool domain_user;
+    DWORD logon_len;
+    TOKEN_SOURCE token_source;
+    LSA_STRING origin_name;
+    void *profile = NULL;
+    DWORD profile_len = 0;
+    LUID logon_id;
+    HANDLE htok = NULL, tmptok;
+    QUOTA_LIMITS quota_limits;
+    NTSTATUS sub_status;
+    PROFILEINFOW profile_info = { 0 };
+    SECURITY_LOGON_TYPE logon_type;
+    TOKEN_USER *usersid = NULL;
+    SID *logon_sid = NULL;
+    TOKEN_GROUPS *extra_groups = NULL;
+    MSV1_0_INTERACTIVE_PROFILE *iprofile;
+    bool found;
+    DWORD len;
+    const char **grpp;
+
+    domain_user = strchr(user, '\\');
+
+    if (interactive && domain_user) {
+	LSA_STRING name;
+	LSA_OPERATIONAL_MODE dummy1;
+
+	/* Interactive, get a token we can use for that. */
+	set_lsa_string(&name, src_module);
+	rv = LsaRegisterLogonProcess(&name, &lsah, &dummy1);
+    } else {
+	/* Just getting information, no need for a token requiring auth. */
+	rv = LsaConnectUntrusted(&lsah);
+    }
+    if (rv)
+	return LsaNtStatusToWinError(rv);
+
+    if (domain_user)
+	set_lsa_string(&package_name, MICROSOFT_KERBEROS_NAME_A);
+    else
+	set_lsa_string(&package_name, MSV1_0_PACKAGE_NAME);
+    rv = LsaLookupAuthenticationPackage(lsah, &package_name, &package_auth);
+    if (rv) {
+	err = LsaNtStatusToWinError(rv);
+	goto out_err;
+    }
+
+    if (domain_user)
+	err = get_kerb_logon(user, password, interactive,
+			     &logon_info, &logon_len, &logon_type);
+    else
+	err = get_local_logon(user, password, &logon_info, &logon_len,
+			      &logon_type);
+    if (err)
+	goto out_err;
+
+    /*
+     * This information is copied into the resulting token.  Note that
+     * SourceName is an 8 character ASCII buffer.
+     */
+    AllocateLocallyUniqueId(&token_source.SourceIdentifier);
+    strncpy(token_source.SourceName, src_module, 7);
+    token_source.SourceName[7] = 0;
+
+    set_lsa_string(&origin_name, src_module);
+
+    /*
+     * Do a first run.  For just query request (not interactive) this
+     * is all we need, but for interactive, we use this to pull
+     * information from for the real logon.
+     *
+     * Pass in NULL for groups so this will generate a new logon sid,
+     * which we may or may not use.
+     */
+    rv = LsaLogonUser(lsah, &origin_name, logon_type, package_auth,
+		      logon_info, logon_len, NULL, &token_source,
+		      &profile, &profile_len, &logon_id, &htok,
+		      &quota_limits, &sub_status);
+    if (rv) {
+	err = LsaNtStatusToWinError(rv);
+	goto out_err;
+    }
+    if (profile)
+	LsaFreeReturnBuffer(profile);
+
+    if (!interactive)
+	/* We have all we need, no setup is necessary. */
+	goto out_finish;
+
+    /*
+     * For interactive, the created handle does not have a proper
+     * logon SID (maybe) and doesn't have the extra groups.  So we
+     * will need to find the right SID and use that.  If the user is
+     * currently logged on, we will pull that logon sid.  If they are
+     * not, we will use the logon SID from the token we just created.
+     */
+
+    /* First find the proper logon sid. */
+    err = read_token_info(htok, TokenUser, (void **) &usersid, NULL);
+    if (err)
+	goto out_err;
+    err = get_logon_sid(usersid->User.Sid, &logon_sid, &found);
+    if (err)
+	goto out_err;
+    if (!found) {
+	/*
+	 * The user isn't logged on, use the logon sid from the token.
+	 * we just created.
+	 */
+	err = get_logon_sid_from_token(htok, &logon_sid, &found);
+	if (err)
+	    goto out_err;
+	if (!found) {
+	    err = ERROR_INVALID_DATA;
+	    goto out_err;
+	}
+    }
+
+    /*
+     * Now add the logon sid to the extra groups.  If it's a network
+     * logon, convert it to interactive by adding the proper sids.
+     */
+    len = sizeof(TOKEN_GROUPS) + sizeof(SID_AND_ATTRIBUTES);
+    for (grpp = add_groups; grpp && *grpp; grpp++)
+	len += sizeof(SID_AND_ATTRIBUTES);
+    for (grpp = groups; grpp && *grpp; grpp++)
+	len += sizeof(SID_AND_ATTRIBUTES);
+    extra_groups = (TOKEN_GROUPS *) malloc(len);
+    if (!extra_groups) {
+	err = STATUS_NO_MEMORY;
+	goto out_err;
+    }
+    memset(extra_groups, 0, len);
+    append_group(extra_groups, logon_sid, NULL,
+		 (SE_GROUP_ENABLED | SE_GROUP_ENABLED_BY_DEFAULT |
+		  SE_GROUP_MANDATORY | SE_GROUP_LOGON_ID));
+    for (grpp = add_groups; grpp && *grpp; grpp++) {
+	err = append_group(extra_groups, NULL, *grpp,
+			   (SE_GROUP_ENABLED |
+			    SE_GROUP_ENABLED_BY_DEFAULT |
+			    SE_GROUP_MANDATORY));
+	if (err)
+	    goto out_err;
+    }
+    for (grpp = groups; grpp && *grpp; grpp++) {
+	err = append_group(extra_groups, NULL, *grpp,
+			   (SE_GROUP_ENABLED |
+			    SE_GROUP_ENABLED_BY_DEFAULT |
+			    SE_GROUP_MANDATORY));
+	if (err)
+	    goto out_err;
+    }
+
+    /* Now get the actual token. */
+    rv = LsaLogonUser(lsah, &origin_name, logon_type, package_auth,
+		      logon_info, logon_len, extra_groups, &token_source,
+		      &profile, &profile_len, &logon_id, &tmptok,
+		      &quota_limits, &sub_status);
+    if (rv) {
+	err = LsaNtStatusToWinError(rv);
+	goto out_err;
+    }
+    CloseHandle(htok);
+    htok = tmptok;
+
+    /* Interactive logons should have the profile loaded. */
+    iprofile = (MSV1_0_INTERACTIVE_PROFILE *) profile;
+    if (iprofile->MessageType == MsV1_0InteractiveProfile)
+	profile_info.lpServerName = iprofile->LogonServer.Buffer;
+
+    profile_info.dwSize = sizeof(profile_info);
+    profile_info.dwFlags = PI_NOUI;
+    wuser = calloc(user_bytes + sizeof(wchar_t), 1);
+    if (!wuser) {
+	err = STATUS_NO_MEMORY;
+	goto out_err;
+    }
+    mbstowcs(wuser, user, user_chars);
+    profile_info.lpUserName = wuser;
+    if (!LoadUserProfileW(htok, &profile_info)) {
+	err = GetLastError();
+	goto out_err;
+    }
+
+ out_finish:
+    if (logon_type != Network) {
+	/*
+	 * We want an impersonation token.  Network tokens do that,
+	 * but others do not.
+	 */
+	if (!DuplicateToken(htok, SecurityImpersonation, &tmptok))
+	    err = GetLastError();
+	CloseHandle(htok);
+	htok = tmptok;
+    }
+
+    *userh = htok;
+    htok = NULL;
+
+ out_err:
+    if (extra_groups)
+	free_groups(extra_groups);
+    if (htok)
+	CloseHandle(htok);
+    if (usersid)
+	free(usersid);
+    if (wuser)
+	free(wuser);
+    if (logon_info)
+	free(logon_info);
+    if (profile_info.hProfile)
+	UnloadUserProfile(htok, profile_info.hProfile);
+    if (profile)
+	LsaFreeReturnBuffer(profile);
+    if (lsah)
+	LsaDeregisterLogonProcess(lsah);
+    LsaClose(lsah);
+
+    return err;
+}
+
+int
+gensio_win_pty_alloc(struct gensio_os_funcs *o,
+		     HANDLE *rreadh, HANDLE *rwriteh,
+		     HANDLE *child_in, HANDLE *child_out,
+		     HPCON *rptyh)
+{
+    HANDLE readh_m = NULL, readh_s = NULL;
+    HANDLE writeh_m = NULL, writeh_s = NULL;
+    HPCON ptyh = NULL;
+    COORD winsize;
+    HRESULT hr;
+    HANDLE imptokh = NULL;
+    int err;
+
+    /*
+     * We can't create pipes or the pseudoconsole with the default
+     * token set because it can't access the default security token.
+     * Briefly go back to the main access token to do this.
+     */
+    if (!OpenThreadToken(GetCurrentThread(),
+			 TOKEN_ALL_ACCESS,
+			 TRUE,
+			 &imptokh)) {
+	if (GetLastError() != ERROR_NO_TOKEN)
+	    goto out_err_conv;
+    } else {
+	if (!RevertToSelf()) {
+	    CloseHandle(imptokh);
+	    imptokh = NULL;
+	    goto out_err_conv;
+	}
+    }
+
+    if (!CreatePipe(&writeh_s, &writeh_m, NULL, 0))
+	goto out_err_conv;
+    if (!SetHandleInformation(writeh_s, HANDLE_FLAG_INHERIT, 0))
+	goto out_err_conv;
+    if (!CreatePipe(&readh_m, &readh_s, NULL, 0))
+	goto out_err_conv;
+    if (!SetHandleInformation(readh_s, HANDLE_FLAG_INHERIT, 0))
+	goto out_err_conv;
+    winsize.X = 80;
+    winsize.Y = 25;
+    hr = CreatePseudoConsole(winsize, writeh_s, readh_s, 0, &ptyh);
+    if (hr != S_OK) {
+	if (HRESULT_FACILITY(hr) == FACILITY_WIN32)
+	    err = gensio_os_err_to_err(o, HRESULT_CODE(hr));
+	else
+	    err = gensio_os_err_to_err(o, hr); /* Force an OS_ERR. */
+	goto out_err;
+    }
+
+    /* Go back to the impersonation token. */
+    if (imptokh) {
+	if (!SetThreadToken(NULL, imptokh))
+	    goto out_err_conv;
+    }
+
+    *child_in = writeh_s;
+    *child_out = readh_s;
+    *rwriteh = writeh_m;
+    *rreadh = readh_m;
+    *rptyh = ptyh;
+
+    return 0;
+
+ out_err_conv:
+    err = gensio_os_err_to_err(o, GetLastError());
+ out_err:
+    if (readh_m)
+	CloseHandle(readh_m);
+    if (readh_s)
+	CloseHandle(readh_s);
+    if (writeh_m)
+	CloseHandle(writeh_m);
+    if (writeh_s)
+	CloseHandle(writeh_s);
+    if (ptyh)
+	ClosePseudoConsole(ptyh);
+    return err;
+}
+
+int
+gensio_win_pty_start(struct gensio_os_funcs *o,
+		     HPCON ptyh, HANDLE *child_in, HANDLE *child_out,
+		     const char **argv, const char **env,
+		     const char *start_dir, HANDLE *child)
+{
+    char *cmdline, *envb = NULL;
+    STARTUPINFOEX si;
+    PROCESS_INFORMATION procinfo;
+    size_t len;
+    HANDLE tokh = NULL, imptokh = NULL;
+    int err;
+    bool setuser = true;
+    bool attrs_added = false;
+
+    memset(&si, 0, sizeof(si));
+
+    err = argv_to_win_cmdline(o, argv, &cmdline);
+    if (err)
+	return err;
+
+    if (env) {
+	envb = win_env_to_block(o, env);
+	if (!envb) {
+	    err = GE_NOMEM;
+	    goto out_err;
+	}
+    }
+
+    si.StartupInfo.cb = sizeof(STARTUPINFOEX);
+
+    /*
+     * This is not in any of the examples on Microsoft's site, but if
+     * the application has stdio relocated, anything that uses stdio
+     * in the child will get the parent's stdio.  If the child opens
+     * the console, all is good, but that's not the normal case.
+     *
+     * So set the stdio to the pipe, too.
+     */
+    si.StartupInfo.hStdInput = *child_in;
+    si.StartupInfo.hStdOutput = *child_out;
+    si.StartupInfo.hStdError = *child_out;
+    si.StartupInfo.dwFlags |= STARTF_USESTDHANDLES;
+
+    InitializeProcThreadAttributeList(NULL, 1, 0, &len);
+    si.lpAttributeList = o->zalloc(o, len);
+    if (!si.lpAttributeList) {
+	err = GE_NOMEM;
+	goto out_err;
+    }
+    if (!InitializeProcThreadAttributeList(si.lpAttributeList, 1, 0, &len))
+	goto out_err_conv;
+    if (!UpdateProcThreadAttribute(si.lpAttributeList,
+                                   0,
+                                   PROC_THREAD_ATTRIBUTE_PSEUDOCONSOLE,
+                                   ptyh,
+                                   sizeof(ptyh),
+                                   NULL,
+                                   NULL))
+	goto out_err_conv;
+    attrs_added = true;
+
+    /*
+     * Get the impersonation token, convert it to a real token, and
+     * create the process as that user.
+     */
+    if (!OpenThreadToken(GetCurrentThread(),
+			 TOKEN_ALL_ACCESS,
+			 TRUE,
+			 &imptokh)) {
+	if (GetLastError() == ERROR_NO_TOKEN)
+	    /* Impersonation token not set, just do a normal create process. */
+	    setuser = false;
+	else
+	    goto out_err_conv;
+    }
+
+    if (setuser) {
+	if (!DuplicateTokenEx(imptokh, 0, NULL, SecurityImpersonation,
+			      TokenPrimary, &tokh))
+	    goto out_err_conv;
+
+	if (!CreateProcessAsUserA(tokh,
+				  NULL,
+				  cmdline,
+				  NULL,
+				  NULL,
+				  FALSE,
+				  (NORMAL_PRIORITY_CLASS |
+				   EXTENDED_STARTUPINFO_PRESENT |
+				   CREATE_NEW_PROCESS_GROUP),
+				  envb,
+				  start_dir,
+				  &si.StartupInfo,
+				  &procinfo))
+	    goto out_err_conv;
+    } else {
+	if (!CreateProcess(NULL,
+			   cmdline,
+			   NULL,
+			   NULL,
+			   FALSE,
+			   (NORMAL_PRIORITY_CLASS |
+			    EXTENDED_STARTUPINFO_PRESENT |
+			    CREATE_NEW_PROCESS_GROUP),
+			   envb,
+			   start_dir,
+			   &si.StartupInfo,
+			   &procinfo))
+	    goto out_err_conv;
+    }
+
+    CloseHandle(*child_in);
+    *child_in = NULL;
+    CloseHandle(*child_out);
+    *child_out = NULL;
+    if (imptokh)
+	CloseHandle(imptokh);
+    if (tokh)
+	CloseHandle(tokh);
+    CloseHandle(procinfo.hThread);
+    DeleteProcThreadAttributeList(si.lpAttributeList);
+    o->free(o, si.lpAttributeList);
+    *child = procinfo.hProcess;
+    return 0;
+
+ out_err_conv:
+    err = gensio_os_err_to_err(o, GetLastError());
+ out_err:
+    if (imptokh)
+	CloseHandle(imptokh);
+    if (tokh)
+	CloseHandle(tokh);
+    if (attrs_added)
+	DeleteProcThreadAttributeList(si.lpAttributeList);
+    if (si.lpAttributeList)
+	o->free(o, si.lpAttributeList);
+    if (cmdline)
+	o->free(o, cmdline);
+    if (envb)
+	o->free(o, envb);
+    return err;
+}
+
 #else /* _WIN32 */
 
 #include <fcntl.h>
@@ -2100,7 +2952,8 @@ extern char **environ;
 int
 gensio_unix_do_exec(struct gensio_os_funcs *o,
 		    const char *argv[], const char **env,
-		    bool stderr_to_stdout,
+		    const char *start_dir,
+		    unsigned int flags,
 		    int *rpid,
 		    int *rin, int *rout, int *rerr)
 {
@@ -2110,7 +2963,7 @@ gensio_unix_do_exec(struct gensio_os_funcs *o,
     int stderrpipe[2] = {-1, -1};
     int pid = -1;
 
-    if (stderr_to_stdout && rerr)
+    if (rerr && (flags & GENSIO_EXEC_STDERR_TO_STDOUT))
 	return GE_INVAL;
 
     err = pipe(stdinpipe);
@@ -2143,7 +2996,7 @@ gensio_unix_do_exec(struct gensio_os_funcs *o,
 
 	dup2(stdinpipe[0], 0);
 	dup2(stdoutpipe[1], 1);
-	if (stderr_to_stdout)
+	if (flags & GENSIO_EXEC_STDERR_TO_STDOUT)
 	    dup2(stdoutpipe[1], 2);
 	else if (rerr)
 	    dup2(stderrpipe[1], 2);
@@ -2152,6 +3005,14 @@ gensio_unix_do_exec(struct gensio_os_funcs *o,
 	for (i = 3; i < openfiles; i++)
 	    close(i);
 
+	if (start_dir) {
+	    if (chdir(start_dir)) {
+		fprintf(stderr, "stdio fork: chdir to %s failed: %s",
+			start_dir, strerror(errno));
+		exit(1);
+	    }
+	}
+
 	err = gensio_unix_os_setupnewprog();
 	if (err) {
 	    fprintf(stderr, "Unable to set groups or user: %s\r\n",
@@ -2197,6 +3058,140 @@ gensio_unix_do_exec(struct gensio_os_funcs *o,
     return err;
 }
 
+int
+gensio_unix_pty_alloc(struct gensio_os_funcs *o, int *rfd)
+{
+    int fd = posix_openpt(O_RDWR | O_NOCTTY);
+    if (fd == -1)
+	return gensio_os_err_to_err(o, errno);
+    *rfd = fd;
+    return 0;
+}
+
+int
+gensio_unix_pty_start(struct gensio_os_funcs *o,
+		      int pfd, const char **argv, const char **env,
+		      const char *start_dir, pid_t *rpid)
+{
+    const char *pgm;
+    pid_t pid = -1;
+    int err;
+
+    if (unlockpt(pfd) < 0)
+	goto out_errno;
+
+    if (!argv)
+	goto skip_child;
+
+    pid = fork();
+    if (pid < 0)
+	goto out_errno;
+
+    if (pid == 0) {
+	/*
+	 * Delay getting the slave until here becase ptsname is not
+	 * thread-safe, but after the fork we are single-threaded.
+	 */
+	char *slave = ptsname(pfd);
+	int i, openfiles = sysconf(_SC_OPEN_MAX);
+	int fd;
+
+	/* Set the owner of the slave PT. */
+	/* FIXME - This should not be necessary, can we remove? */
+#if 0
+	if (grantpt(pfd) < 0)
+	    exit(1);
+#endif
+
+	if (start_dir) {
+	    if (chdir(start_dir)) {
+		fprintf(stderr, "pty fork: chdir to %s failed: %s",
+			start_dir, strerror(errno));
+		exit(1);
+	    }
+	}
+
+	if (setsid() == -1) {
+	    fprintf(stderr, "pty fork: failed to start new session: %s\r\n",
+		    strerror(errno));
+	    exit(1);
+	}
+
+#if 0 /* FIXME = do we need this? */
+	if (setpgid(0, 0) == -1) {
+	    fprintf(stderr, "pty fork: failed setpgid: %s\r\n",
+		    strerror(errno));
+	    exit(1);
+	}
+#endif
+
+	fd = open(slave, O_RDWR);
+	if (fd == -1) {
+	    fprintf(stderr, "pty fork: failed to open slave terminal: %s\r\n",
+		    strerror(errno));
+	    exit(1);
+	}
+
+#if defined(TIOCSCTTY) && !defined(linux)
+	/* Linux sets the first opened TTY to the controlling terminal. */
+	if (ioctl(fd, TIOCSCTTY, NULL) == -1) {
+	    fprintf(stderr, "pty fork: failed to set controlling tty: %s\r\n",
+		    strerror(errno));
+	    exit(1);
+	}
+#endif
+
+	/* fd will be closed by the loop to close everything. */
+	if (open("/dev/tty", O_RDWR) == -1) {
+	    fprintf(stderr, "pty fork: failed to set control term: %s\r\n",
+		    strerror(errno));
+	    exit(1);
+	}
+
+	if (dup2(fd, 0) == -1) {
+	    fprintf(stderr, "pty fork: stdin open fail\r\n");
+	    exit(1);
+	}
+
+	if (dup2(fd, 1) == -1) {
+	    fprintf(stderr, "pty fork: stdout open fail\r\n");
+	    exit(1);
+	}
+
+	if (dup2(fd, 2) == -1) {
+	    fprintf(stderr, "pty fork: stderr open fail\r\n");
+	    exit(1);
+	}
+
+	/* Close everything. */
+	for (i = 3; i < openfiles; i++)
+	    close(i);
+
+	err = gensio_unix_os_setupnewprog();
+	if (err) {
+	    fprintf(stderr, "Unable to set groups or user: %s\r\n",
+		    strerror(err));
+	    exit(1);
+	}
+
+	if (env)
+	    environ = (char **) env;
+
+	pgm = argv[0];
+	if (*pgm == '-')
+	    pgm++;
+	execvp(pgm, (char **) argv);
+	fprintf(stderr, "Unable to exec %s: %s\r\n", argv[0],
+		strerror(errno));
+	exit(1); /* Only reached on error. */
+    }
+ skip_child:
+    *rpid = pid;
+    return 0;
+ out_errno:
+    return gensio_os_err_to_err(o, errno);
+}
+
 #endif /* _WIN32 */
 
 #ifdef ENABLE_INTERNAL_TRACE
