@@ -363,10 +363,6 @@ gensio_unix_unlock(struct gensio_lock *lock)
     UNLOCK(&lock->lock);
 }
 
-struct gensio_iod_dev {
-    struct gensio_unix_termios *termios;
-};
-
 struct gensio_iod_file {
     struct gensio_lock *lock;
     struct gensio_runner *runner;
@@ -380,8 +376,16 @@ struct gensio_iod_socket {
     void *sockinfo;
 };
 
+struct gensio_iod_pty {
+    const char **argv;
+    const char **env;
+    pid_t pid;
+    char *start_dir;
+};
+
 struct gensio_iod_unix {
     struct gensio_iod r;
+    int orig_fd;
     int fd;
     enum gensio_iod_type type;
     bool handlers_set;
@@ -394,10 +398,13 @@ struct gensio_iod_unix {
 
     struct stdio_mode *mode;
 
+    /* Used by dev and pty. */
+    struct gensio_unix_termios *termios;
+
     union {
-	struct gensio_iod_dev dev;
 	struct gensio_iod_file file;
 	struct gensio_iod_socket socket;
+	struct gensio_iod_pty pty;
     } u;
 };
 
@@ -1016,11 +1023,14 @@ gensio_handle_fork(struct gensio_os_funcs *f)
 
 static int
 gensio_unix_add_iod(struct gensio_os_funcs *o, enum gensio_iod_type type,
-		    intptr_t fd, struct gensio_iod **riod)
+		    intptr_t ofd, struct gensio_iod **riod)
 {
     struct gensio_iod_unix *iod = NULL;
     bool closefd = false;
-    int err = GE_NOMEM;
+    int err = GE_NOMEM, fd = ofd;
+
+    if (type >= NR_GENSIO_IOD_TYPES)
+	return GE_INVAL;
 
     if (type == GENSIO_IOD_CONSOLE) {
 	if (fd == 0)
@@ -1032,6 +1042,11 @@ gensio_unix_add_iod(struct gensio_os_funcs *o, enum gensio_iod_type type,
 	if (fd == -1)
 	    return gensio_os_err_to_err(o, errno);
 	closefd = true;
+    } else if (type == GENSIO_IOD_PTY) {
+	err = gensio_unix_pty_alloc(o, &fd);
+	if (err)
+	    return err;
+	closefd = true;
     }
 
     iod = o->zalloc(o, sizeof(*iod));
@@ -1041,6 +1056,7 @@ gensio_unix_add_iod(struct gensio_os_funcs *o, enum gensio_iod_type type,
     }
     iod->r.f = o;
     iod->fd = fd;
+    iod->orig_fd = ofd;
     if (type == GENSIO_IOD_STDIO) {
 	struct stat statb;
 
@@ -1060,6 +1076,8 @@ gensio_unix_add_iod(struct gensio_os_funcs *o, enum gensio_iod_type type,
 	    err = GE_INVAL;
 	    goto out_err;
 	}
+    } else if (type == GENSIO_IOD_PTY) {
+	iod->u.pty.pid = -1;
     }
     iod->type = type;
 
@@ -1089,13 +1107,22 @@ static void
 gensio_unix_release_iod(struct gensio_iod *iiod)
 {
     struct gensio_iod_unix *iod = i_to_sel(iiod);
+    struct gensio_os_funcs *o = iod->r.f;
 
     assert(!iod->handlers_set);
     if (iod->type == GENSIO_IOD_FILE) {
-	iod->r.f->free_runner(iod->u.file.runner);
-	iod->r.f->free_lock(iod->u.file.lock);
+	o->free_runner(iod->u.file.runner);
+	o->free_lock(iod->u.file.lock);
     }
-    iod->r.f->free(iiod->f, iod);
+    if (iod->type == GENSIO_IOD_PTY) {
+	if (iod->u.pty.argv)
+	    gensio_argv_free(o, iod->u.pty.argv);
+	if (iod->u.pty.env)
+	    gensio_argv_free(o, iod->u.pty.env);
+	if (iod->u.pty.start_dir)
+	    o->free(o, iod->u.pty.start_dir);
+    }
+    o->free(o, iod);
 }
 
 static int
@@ -1194,14 +1221,14 @@ gensio_unix_close(struct gensio_iod **iodp)
     if (iod->type != GENSIO_IOD_FILE)
 	gensio_unix_do_cleanup_nonblock(o, iod->fd, &iod->mode);
 
-    if (iod->type == GENSIO_IOD_DEV ||
-		(iod->type == GENSIO_IOD_CONSOLE && iod->fd == 0))
-	gensio_unix_cleanup_termios(o, &iod->u.dev.termios, iod->fd);
+    if (iod->termios)
+	gensio_unix_cleanup_termios(o, &iod->termios, iod->fd);
 
     if (iod->type == GENSIO_IOD_SOCKET) {
 	err = o->close_socket(iiod, false, true);
-    } else if (!iod->is_stdio) {
+    } else if (!iod->is_stdio && iod->fd != -1) {
 	err = close(iod->fd);
+	iod->fd = -1;
 #ifdef ENABLE_INTERNAL_TRACE
 	/* Close should never fail, but don't crash in production builds. */
 	assert(err == 0);
@@ -1263,10 +1290,10 @@ gensio_unix_makeraw(struct gensio_iod *iiod)
 {
     struct gensio_iod_unix *iod = i_to_sel(iiod);
 
-    if (iod->type == GENSIO_IOD_DEV ||
-		(iod->type == GENSIO_IOD_CONSOLE && iod->fd == 0))
+    if (iod->type == GENSIO_IOD_DEV || iod->type == GENSIO_IOD_PTY ||
+		(iod->type == GENSIO_IOD_CONSOLE && iod->orig_fd == 0))
 	/* Only set this for stdin or other files. */
-	return gensio_unix_setup_termios(iiod->f, iod->fd, &iod->u.dev.termios);
+	return gensio_unix_setup_termios(iiod->f, iod->fd, &iod->termios);
 
     return 0;
 }
@@ -1297,7 +1324,8 @@ gensio_unix_open_dev(struct gensio_os_funcs *o, const char *name,
 static int
 gensio_unix_exec_subprog(struct gensio_os_funcs *o,
 			 const char *argv[], const char **env,
-			 bool stderr_to_stdout,
+			 const char *start_dir,
+			 unsigned int flags,
 			 intptr_t *rpid,
 			 struct gensio_iod **rstdin,
 			 struct gensio_iod **rstdout,
@@ -1308,7 +1336,7 @@ gensio_unix_exec_subprog(struct gensio_os_funcs *o,
     struct gensio_iod *stdiniod = NULL, *stdoutiod = NULL, *stderriod = NULL;
     int pid = -1;
 
-    err = gensio_unix_do_exec(o, argv, env, stderr_to_stdout, &pid, &infd,
+    err = gensio_unix_do_exec(o, argv, env, start_dir, flags, &pid, &infd,
 			      &outfd, rstderr ? &errfd : NULL);
     if (err)
 	return err;
@@ -1361,6 +1389,88 @@ gensio_unix_exec_subprog(struct gensio_os_funcs *o,
     return err;
 }
 
+static int
+gensio_unix_pty_control(struct gensio_iod_unix *iod, int op, bool get,
+			intptr_t val)
+{
+    struct gensio_os_funcs *o = iod->r.f;
+    int err = 0;
+    const char **nargv;
+
+    if (get) {
+	if (op == GENSIO_IOD_CONTROL_PID) {
+	    if (iod->u.pty.pid == -1)
+		return GE_NOTREADY;
+	    *((intptr_t *) val) = iod->u.pty.pid;
+	    return 0;
+	}
+	return GE_NOTSUP;
+    }
+
+    switch (op) {
+    case GENSIO_IOD_CONTROL_ARGV:
+	err = gensio_argv_copy(o, (const char **) val, NULL, &nargv);
+	if (err)
+	    return err;
+	if (iod->u.pty.argv)
+	    gensio_argv_free(o, iod->u.pty.argv);
+	iod->u.pty.argv = nargv;
+	return 0;
+
+    case GENSIO_IOD_CONTROL_ENV:
+	err = gensio_argv_copy(o, (const char **) val, NULL, &nargv);
+	if (err)
+	    return err;
+	if (iod->u.pty.env)
+	    gensio_argv_free(o, iod->u.pty.env);
+	iod->u.pty.env = nargv;
+	return 0;
+
+    case GENSIO_IOD_CONTROL_START:
+	return gensio_unix_pty_start(o, iod->fd, iod->u.pty.argv,
+				     iod->u.pty.env, iod->u.pty.start_dir,
+				     &iod->u.pty.pid);
+
+    case GENSIO_IOD_CONTROL_STOP:
+	if (iod->fd != -1) {
+	    close(iod->fd);
+	    iod->fd = -1;
+	}
+	return 0;
+
+    case GENSIO_IOD_CONTROL_WIN_SIZE: {
+	struct winsize win;
+	struct gensio_winsize *gwin = (struct gensio_winsize *) val;
+
+	win.ws_row = gwin->ws_row;
+	win.ws_col = gwin->ws_col;
+	win.ws_xpixel = gwin->ws_xpixel;
+	win.ws_ypixel = gwin->ws_ypixel;
+	if (ioctl(iod->fd, TIOCSWINSZ, &win) == -1)
+	    err = gensio_os_err_to_err(o, errno);
+	return err;
+    }
+
+    case GENSIO_IOD_CONTROL_START_DIR: {
+	char *dir = (char *) val;
+
+	if (dir) {
+	    dir = gensio_strdup(o, dir);
+	    if (!dir)
+		return GE_NOMEM;
+	}
+
+	if (iod->u.pty.start_dir)
+	    o->free(o, iod->u.pty.start_dir);
+	iod->u.pty.start_dir = dir;
+	return 0;
+    }
+
+    default:
+	return GE_NOTSUP;
+    }
+}
+
 static int
 gensio_unix_iod_control(struct gensio_iod *iiod, int op, bool get, intptr_t val)
 {
@@ -1378,11 +1488,14 @@ gensio_unix_iod_control(struct gensio_iod *iiod, int op, bool get, intptr_t val)
 	return 0;
     }
 
+    if (iod->type == GENSIO_IOD_PTY)
+	return gensio_unix_pty_control(iod, op, get, val);
+
     if (iod->type != GENSIO_IOD_DEV)
 	return GE_NOTSUP;
 
     return gensio_unix_termios_control(iiod->f, op, get, val,
-				       &iod->u.dev.termios, iod->fd);
+				       &iod->termios, iod->fd);
 }
 
 static int
@@ -1399,7 +1512,7 @@ gensio_unix_kill_subprog(struct gensio_os_funcs *o, intptr_t pid,
 
 static int
 gensio_unix_wait_subprog(struct gensio_os_funcs *o, intptr_t pid,
-			      int *retcode)
+			 int *retcode)
 {
     pid_t rv;
 
