@@ -35,6 +35,7 @@ struct stel_req {
 };
 
 struct stel_data {
+    struct gensio *io;
     struct sergensio *sio;
 
     struct gensio_os_funcs *o;
@@ -497,7 +498,7 @@ stelc_com_port_cmd(void *handler_data, const unsigned char *option,
     char *sig = NULL;
     unsigned int sig_len;
     gensiods vlen = sizeof(int);
-    struct gensio *io = sergensio_get_my_gensio(sdata->sio);
+    struct gensio *io = sdata->io;
 
     if (len < 2)
 	return;
@@ -656,6 +657,8 @@ stel_free(void *handler_data)
 {
     struct stel_data *sdata = handler_data;
 
+    if (sdata->sio)
+	sergensio_data_free(sdata->sio);
     if (sdata->lock)
 	sdata->o->free_lock(sdata->lock);
     while (sdata->reqs) {
@@ -664,8 +667,6 @@ stel_free(void *handler_data)
 	sdata->reqs = req->next;
 	sdata->o->free(sdata->o, req);
     }
-    if (sdata->sio)
-	sergensio_data_free(sdata->sio);
     sdata->o->free(sdata->o, sdata);
 }
 
@@ -693,7 +694,7 @@ stels_cb_com_port_will_do(void *handler_data, unsigned char cmd)
 	sdata->do_2217 = sdata->allow_2217;
 
     if (!sdata->reported_modemstate && sdata->do_2217) {
-	struct gensio *io = sergensio_get_my_gensio(sdata->sio);
+	struct gensio *io = sdata->io;
 
 	if (gensio_get_cb(io)) {
 	    int val = 255;
@@ -723,7 +724,7 @@ stels_cb_com_port_cmd(void *handler_data, const unsigned char *option,
     struct stel_data *sdata = handler_data;
     int val = 0;
     gensiods vlen = sizeof(int);
-    struct gensio *io = sergensio_get_my_gensio(sdata->sio);
+    struct gensio *io = sdata->io;
 
     if (len < 2)
 	return;
@@ -835,7 +836,7 @@ stels_cb_com_port_cmd(void *handler_data, const unsigned char *option,
 	gensio_cb(io, GENSIO_EVENT_SER_MODEMSTATE, 0,
 		  (unsigned char *) &val, &vlen, NULL);
 	break;
-	
+
     case 12:
 	if (len < 3)
 	    return;
@@ -853,7 +854,7 @@ static void
 stels_got_cmd(void *handler_data, unsigned char cmd)
 {
     struct stel_data *sdata = handler_data;
-    struct gensio *io = sergensio_get_my_gensio(sdata->sio);
+    struct gensio *io = sdata->io;
 
     if (cmd == TN_BREAK)
 	gensio_cb(io, GENSIO_EVENT_SEND_BREAK, 0, NULL, NULL, NULL);
@@ -863,7 +864,7 @@ static void
 stels_cb_got_sync(void *handler_data)
 {
     struct stel_data *sdata = handler_data;
-    struct gensio *io = sergensio_get_my_gensio(sdata->sio);
+    struct gensio *io = sdata->io;
 
     gensio_cb(io, GENSIO_EVENT_SER_SYNC, 0, NULL, NULL, NULL);
 }
@@ -875,7 +876,7 @@ stels_timeout(void *handler_data)
 
     stel_lock(sdata);
     if (!sdata->reported_modemstate && sdata->do_2217) {
-	struct gensio *io = sergensio_get_my_gensio(sdata->sio);
+	struct gensio *io = sdata->io;
 	int val = 255;
 	gensiods vlen = sizeof(val);
 
@@ -992,14 +993,11 @@ telnet_gensio_alloc(struct gensio *child, const char * const args[],
     if (!io)
 	goto out_nomem;
 
-    gensio_set_is_reliable(io, gensio_is_reliable(child));
-
-    sdata->sio = sergensio_data_alloc(o, io, sergensio_stel_func, sdata);
-    if (!sdata->sio)
-	goto out_nomem;
+    sdata->io = io;
 
     if (sdata->allow_2217) {
-	err = gensio_addclass(io, "sergensio", sdata->sio);
+	err = sergensio_addclass(o, io, sergensio_stel_func, sdata,
+				 &sdata->sio);
 	if (err)
 	    goto out_err;
     }
@@ -1150,14 +1148,11 @@ stela_finish_parent(void *acc_data, void *finish_data, struct gensio *io,
     struct stel_data *sdata = finish_data;
     int err;
 
-    gensio_set_is_reliable(io, gensio_is_reliable(child));
-    sdata->sio = sergensio_data_alloc(sdata->o, io, sergensio_stel_func,
-				      sdata);
-    if (!sdata->sio)
-	return GE_NOMEM;
+    sdata->io = io;
 
     if (sdata->allow_2217) {
-	err = gensio_addclass(io, "sergensio", sdata->sio);
+	err = sergensio_addclass(sdata->o, io, sergensio_stel_func, sdata,
+				 &sdata->sio);
 	if (err)
 	    return err;
     }
@@ -1252,14 +1247,8 @@ telnet_gensio_accepter_alloc(struct gensio_accepter *child,
 	goto out_err;
 
     if (allow_2217) {
-	stela->sacc = sergensio_acc_data_alloc(o, accepter,
-					       sergensio_stela_func, stela);
-	if (!stela->sacc) {
-	    err = GE_NOMEM;
-	    goto out_err;
-	}
-
-	err = gensio_acc_addclass(accepter, "sergensio", stela->sacc);
+	err = sergensio_acc_addclass(o, accepter, sergensio_stela_func, stela,
+				     &stela->sacc);
 	if (err)
 	    goto out_err;
     }
