@@ -133,10 +133,10 @@ open_tempfile(char *name, unsigned int len, const char *pattern)
 #else /* _WIN32 */
 
 #include <sys/types.h>
+#include <sys/wait.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <fcntl.h>
-#include <sys/wait.h>
 bool
 file_is_accessible_dev(const char *filename)
 {
@@ -180,25 +180,23 @@ open_tempfile(char *name, unsigned int len, const char *pattern)
 #endif
 
 bool sleep_on_timeout_err;
+#ifndef _WIN32
+bool kill_on_timeout_err;
+#endif
+struct oom_test_data;
+
+static void handle_timeout_err(struct oom_test_data *od);
 
 static void
-handle_timeout_err(void)
-{
-    while (sleep_on_timeout_err)
-	sleep(100);
-    assert(0);
-}
-
-static void
-l_assert_or_stop(bool val, char *expr, int line)
+l_assert_or_stop(struct oom_test_data *od, bool val, char *expr, int line)
 {
     if (val)
 	return;
     fprintf(stderr, "Assert '%s' failed on line %d\n", expr, line);
     fflush(stderr);
-    handle_timeout_err();
+    handle_timeout_err(od);
 }
-#define assert_or_stop(val) l_assert_or_stop(val, #val, __LINE__)
+#define assert_or_stop(od, val) l_assert_or_stop(od, val, #val, __LINE__)
 
 #if HAVE_LIBSCTP
 #include <sys/socket.h>
@@ -234,7 +232,7 @@ get_echo_dev(struct gensio_os_funcs *o, const char *testname,
 	if (strlen(te) == 0) {
 	    printf("Serial echo device disabled, skipping serialdev test\n");
 	    if (te)
-		o->free(o, te);
+		gensio_os_funcs_zfree(o, te);
 	    return false;
 	}
 	e = te;
@@ -249,12 +247,12 @@ get_echo_dev(struct gensio_os_funcs *o, const char *testname,
 	printf("Serial echo device '%s' doesn't exist or is not accessible,\n"
 	       "skipping %s test\n", e, testname);
 	if (te)
-	    o->free(o, te);
+	    gensio_os_funcs_zfree(o, te);
 	return false;
     }
     *newstr = gensio_alloc_sprintf(o, str, e);
     if (te)
-	o->free(o, te);
+	gensio_os_funcs_zfree(o, te);
     if (!*newstr) {
 	printf("Unable to allocate memory for echo device '%s',\n"
 	       "skipping %s test\n", e, testname);
@@ -315,7 +313,7 @@ ipmisim_cb(struct gensio *io, void *user_data, int event, int err,
 
 	if (verbose)
 	    printf("IPMISIM err: %s\n", gensio_err_to_str(err));
-	test->o->wake(test->waiter);
+	gensio_os_funcs_wake(test->o, test->waiter);
 	return 0;
     }
 
@@ -340,7 +338,7 @@ ipmisim_cb(struct gensio *io, void *user_data, int event, int err,
 	    if (test->wait_pos >= sizeof(waitstr)) {
 		test->wait_pos = 0;
 		test->str_found = true;
-		test->o->wake(test->waiter);
+		gensio_os_funcs_wake(test->o, test->waiter);
 		break;
 	    }
 	} else {
@@ -363,7 +361,7 @@ ipmisim_err_cb(struct gensio *io, void *user_data, int event, int err,
 	gensio_set_read_callback_enable(io, false);
 	if (verbose)
 	    printf("IPMISIM err err: %s\n", gensio_err_to_str(err));
-	test->o->wake(test->waiter);
+	gensio_os_funcs_wake(test->o, test->waiter);
 	return 0;
     }
 
@@ -429,7 +427,8 @@ ipmisim_start(struct gensio_os_funcs *o, struct oom_tests *test)
     gensio_set_read_callback_enable(test->io2, true);
 
  retry:
-    err = o->wait_intr_sigmask(test->waiter, 1, &timeout, proc_data);
+    err = gensio_os_funcs_wait_intr_sigmask(o, test->waiter,
+					    1, &timeout, proc_data);
     if (err == GE_INTERRUPTED)
 	goto retry;
     if (test->err)
@@ -492,11 +491,11 @@ ipmisim_finish(struct gensio_os_funcs *o, struct oom_tests *test)
 	test->configname[0] = '\0';
     }
     if (test->args) {
-	o->free(o, test->args);
+	gensio_os_funcs_zfree(o, test->args);
 	test->args = NULL;
     }
     if (test->waiter) {
-	o->free_waiter(test->waiter);
+	gensio_os_funcs_free_waiter(o, test->waiter);
 	test->waiter = NULL;
     }
     test->o = NULL;
@@ -524,7 +523,7 @@ check_ipmisim_present(struct gensio_os_funcs *o, struct oom_tests *test)
 
     if (!get_echo_dev(o, "ipmisol", ipmisim_config, &config)) {
 	if (tprog)
-	    o->free(o, tprog);
+	    gensio_os_funcs_zfree(o, tprog);
 	return false;
     }
 
@@ -538,7 +537,7 @@ check_ipmisim_present(struct gensio_os_funcs *o, struct oom_tests *test)
     }
     len = strlen(config);
     rv = fwrite(config, 1, len, f);
-    o->free(o, config);
+    gensio_os_funcs_zfree(o, config);
     config = NULL;
     fclose(f);
     if (rv != len) {
@@ -574,7 +573,7 @@ check_ipmisim_present(struct gensio_os_funcs *o, struct oom_tests *test)
 	goto out_err;
     }
 
-    test->waiter = o->alloc_waiter(o);
+    test->waiter = gensio_os_funcs_alloc_waiter(o);
     if (!test->waiter) {
 	printf("Unable to allocate ipmi_sim ewaiter,\n"
 	       "skipping ipmisol test\n");
@@ -587,15 +586,15 @@ check_ipmisim_present(struct gensio_os_funcs *o, struct oom_tests *test)
 	goto out_err;
 
     if (tprog)
-	o->free(o, tprog);
+	gensio_os_funcs_zfree(o, tprog);
 
     return true;
 
  out_err:
     if (tprog)
-	o->free(o, tprog);
+	gensio_os_funcs_zfree(o, tprog);
     if (config)
-	o->free(o, config);
+	gensio_os_funcs_zfree(o, config);
     ipmisim_end(o, test);
     ipmisim_finish(o, test);
 
@@ -658,10 +657,16 @@ struct oom_tests oom_tests[] = {
       /* In this tests some errors will not result in a failure. */
       .allow_no_err_on_trig = true,
       .check_done = 1, .check_value = HAVE_OPENSSL },
-    { "echo", NULL },
-    { "tcp,localhost,", "tcp,0" },
+    { "echo", NULL,
+      .allow_no_err_on_trig = true,
+    },
+    { "tcp,localhost,", "tcp,0",
+      .allow_no_err_on_trig = true,
+    },
     { "sctp,localhost,", "sctp,0",
-      .check_if_present = check_sctp_present, .check_value = HAVE_LIBSCTP },
+      .check_if_present = check_sctp_present, .check_value = HAVE_LIBSCTP,
+      .allow_no_err_on_trig = true,
+    },
     { "udp,ipv4,localhost,", "udp,ipv4,0",
       /* In this tests some errors will not result in a failure. */
       .allow_no_err_on_trig = true
@@ -675,17 +680,26 @@ struct oom_tests oom_tests[] = {
        * large I/O sizes.  So limit it to a reasonable value.
        */
       .max_io_size = 10000 },
-    { "telnet(rfc2217),tcp,localhost,", "telnet(rfc2217),tcp,0" },
+    { "telnet(rfc2217),tcp,localhost,", "telnet(rfc2217),tcp,0",
+      .allow_no_err_on_trig = true,
+    },
     { "serialdev,%s,115200", NULL,
       .check_if_present = check_serialdev_present,
+      .allow_no_err_on_trig = true,
       /*
        * The error injections cause this to take way to long with
        * large I/O sizes.  So limit it to a reasonable value.
        */
       .max_io_size = 1000 },
-    { "telnet,tcp,localhost,", "telnet,tcp,0" },
-    { "stdio,cat", NULL },
-    { "conacc,tcp,localhost,", "tcp,0", .conacc=true },
+    { "telnet,tcp,localhost,", "telnet,tcp,0",
+      .allow_no_err_on_trig = true,
+    },
+    { "stdio,cat", NULL,
+      .allow_no_err_on_trig = true,
+    },
+    { "conacc,tcp,localhost,", "tcp,0", .conacc=true,
+      .allow_no_err_on_trig = true,
+    },
     { "serialdev,", "conacc,pty(raw)",
       /* In this tests some errors will not result in a failure. */
       .allow_no_err_on_trig = true,
@@ -706,6 +720,11 @@ struct oom_tests oom_tests[] = {
        */
       .no_default_run = true,
       .check_value = HAVE_PTY },
+    { "ax25(laddr=test-2,addr=\"0,test-1,test-2\"),kiss(writebuf=512,readbuf=512),tcp,localhost,",
+      "ax25(laddr=test-1),kiss(writebuf=512,readbuf=512),tcp,0",
+      /* In this tests some errors will not result in a failure. */
+      .allow_no_err_on_trig = true,
+    },
     { NULL }
 };
 
@@ -717,7 +736,7 @@ gensio_loop(void *info)
 {
     struct gensio_waiter *closewaiter = info;
 
-    o->wait(closewaiter, 1, NULL);
+    gensio_os_funcs_wait(o, closewaiter, 1, NULL);
 }
 
 static void
@@ -758,8 +777,6 @@ oom_err_to_str(int err)
     }
 }
 
-struct oom_test_data;
-
 struct io_test_data {
     struct gensio *io;
     gensiods write_pos;
@@ -825,11 +842,35 @@ struct ref_trace {
 } ref_trace[512];
 unsigned int ref_trace_pos;
 
+static void
+handle_timeout_err(struct oom_test_data *od)
+{
+#ifndef _WIN32
+    if (kill_on_timeout_err && od->ccon_stderr_io) {
+	int rv;
+	char str[10];
+	gensiods len = sizeof(str);
+
+	rv = gensio_control(od->ccon_stderr_io, GENSIO_CONTROL_DEPTH_FIRST,
+			    GENSIO_CONTROL_GET,
+			    GENSIO_CONTROL_REMOTE_ID, str, &len);
+	if (!rv) {
+	    pid_t pid = strtoul(str, NULL, 0);
+
+	    kill(pid, SIGSEGV);
+	}
+    }
+#endif
+    while (sleep_on_timeout_err)
+	sleep(100);
+    assert(0);
+}
+
 static void
 add_ref_trace(enum ref_trace_op op, unsigned int count, int line,
 	      unsigned int data)
 {
-    o->get_monotonic_time(o, &ref_trace[ref_trace_pos].time);
+    gensio_os_funcs_get_monotonic_time(o, &ref_trace[ref_trace_pos].time);
     ref_trace[ref_trace_pos].op = op;
     ref_trace[ref_trace_pos].refcount = count;
     ref_trace[ref_trace_pos].line = line;
@@ -866,7 +907,7 @@ i_od_deref_and_unlock(struct oom_test_data *od, int line)
 	    gensio_free(od->scon.io);
 	if (od->ccon_stderr_io)
 	    gensio_free(od->ccon_stderr_io);
-	o->free_waiter(od->waiter);
+	gensio_os_funcs_free_waiter(o, od->waiter);
 	od->finished = true;
 	od->next = old_ods;
 	old_ods = od;
@@ -881,7 +922,7 @@ cleanup_ods(void)
 
     while (od) {
 	next = od->next;
-	o->free(o, od);
+	gensio_os_funcs_zfree(o, od);
 	od = next;
     }
 }
@@ -918,7 +959,7 @@ ccon_stderr_closed(struct gensio *io, void *close_data)
 	od->ccon_exit_code_set = true;
     }
     od->ccon_stderr_io = NULL;
-    o->wake(od->waiter);
+    gensio_os_funcs_wake(o, od->waiter);
     gensio_free(io);
     od_deref_and_unlock(od);
 }
@@ -935,7 +976,7 @@ con_closed(struct gensio *io, void *close_data)
     id->closed = true;
     gensio_free(io);
     id->io = NULL;
-    o->wake(od->waiter);
+    gensio_os_funcs_wake(o, od->waiter);
     od_deref_and_unlock(od);
 }
 
@@ -948,7 +989,7 @@ acc_closed(struct gensio_accepter *acc, void *close_data)
     assert(acc == od->acc);
     LOCK(&od->lock);
     od->acc = NULL;
-    o->wake(od->waiter);
+    gensio_os_funcs_wake(o, od->waiter);
     gensio_acc_free(acc);
     od_deref_and_unlock(od);
 }
@@ -988,7 +1029,8 @@ con_cb(struct gensio *io, void *user_data,
     add_ref_trace(ref_inc, err, __LINE__, event);
     assert(id->io == io);
     if (err) {
-	assert_or_stop(!debug || err == GE_REMCLOSE || err == GE_NOTREADY || err == GE_LOCALCLOSED);
+	assert_or_stop(od, !debug || err == GE_REMCLOSE || err == GE_NOTREADY
+		       || err == GE_LOCALCLOSED);
 	gensio_set_write_callback_enable(io, false);
 	gensio_set_read_callback_enable(io, false);
 	if (!id->expect_close || err != GE_REMCLOSE) {
@@ -1000,7 +1042,7 @@ con_cb(struct gensio *io, void *user_data,
 	} else {
 	    id->got_end = true;
 	}
-	o->wake(od->waiter);
+	gensio_os_funcs_wake(o, od->waiter);
 	goto out;
     }
 
@@ -1016,7 +1058,7 @@ con_cb(struct gensio *io, void *user_data,
 		   (long) id->read_pos, (long) *buflen, buf);
 	    fflush(stdout);
 	    assert(!debug);
-	    o->wake(od->waiter);
+	    gensio_os_funcs_wake(o, od->waiter);
 	    goto out_leave_read;
 	}
 
@@ -1025,12 +1067,12 @@ con_cb(struct gensio *io, void *user_data,
 	    gensio_set_write_callback_enable(io, false);
 	    gensio_set_read_callback_enable(io, false);
 	    id->err = OOME_DATA_MISMATCH;
-	    o->wake(od->waiter);
+	    gensio_os_funcs_wake(o, od->waiter);
 	}
 
 	id->read_pos += *buflen;
 	if (id->read_pos >= od->io_size)
-	    o->wake(od->waiter);
+	    gensio_os_funcs_wake(o, od->waiter);
     out_leave_read:
 	id->in_read = false;
 	break;
@@ -1061,13 +1103,13 @@ con_cb(struct gensio *io, void *user_data,
 		gensio_set_write_callback_enable(io, false);
 		gensio_set_read_callback_enable(io, false);
 		id->err = rv;
-		o->wake(od->waiter);
+		gensio_os_funcs_wake(o, od->waiter);
 	    } else {
 		id->write_pos += count;
 	    }
 	} else {
 	    gensio_set_write_callback_enable(io, false);
-	    o->wake(od->waiter);
+	    gensio_os_funcs_wake(o, od->waiter);
 	}
 	id->in_write = false;
 	break;
@@ -1153,7 +1195,7 @@ ccon_stderr_cb(struct gensio *io, void *user_data,
 	od->stderr_rem_closed = true;
 	if (!od->stderr_expect_close || err != GE_REMCLOSE)
 	    od->ccon.err = err;
-	o->wake(od->waiter);
+	gensio_os_funcs_wake(o, od->waiter);
 	OOMUNLOCK(&od->lock);
 	return 0;
     }
@@ -1198,7 +1240,7 @@ ccon_stderr_cb(struct gensio *io, void *user_data,
 			    fflush(stdout);
 			}
 			od->invalid_port_data = true;
-			o->wake(od->waiter);
+			gensio_os_funcs_wake(o, od->waiter);
 			return 0;
 		    }
 		    s = strchr(od->ccon_stderr, ':');
@@ -1217,7 +1259,7 @@ ccon_stderr_cb(struct gensio *io, void *user_data,
 		od->ccon_stderr_pos = size;
 		if (done) {
 		    od->look_for_port = false;
-		    o->wake(od->waiter);
+		    gensio_os_funcs_wake(o, od->waiter);
 		}
 		if (od->ccon_stderr_pos)
 		    goto more_data;
@@ -1244,7 +1286,7 @@ ccon_stderr_open_done(struct gensio *io, int err, void *open_data)
     if (err) {
 	assert(!debug || err == GE_REMCLOSE);
 	od->ccon.err = err;
-	o->wake(od->waiter);
+	gensio_os_funcs_wake(o, od->waiter);
     } else {
 	gensio_set_read_callback_enable(io, true);
     }
@@ -1261,7 +1303,7 @@ scon_open_done(struct gensio *io, int err, void *open_data)
     assert(!od->finished);
     OOMLOCK(&od->lock);
     assert(!id->open_done);
-    o->wake(od->waiter);
+    gensio_os_funcs_wake(o, od->waiter);
     if (id->closed)
 	goto out_unlock;
 
@@ -1271,7 +1313,7 @@ scon_open_done(struct gensio *io, int err, void *open_data)
 		   id->iostr);
 	    fflush(stdout);
 	}
-	assert_or_stop(!debug || err == GE_REMCLOSE || err == GE_INVAL ||
+	assert_or_stop(od, !debug || err == GE_REMCLOSE || err == GE_INVAL ||
 		       err == GE_SHUTDOWN || err == GE_LOCALCLOSED ||
 		       err == GE_NOTREADY);
 	if (err == GE_INVAL)
@@ -1300,7 +1342,7 @@ ccon_open_done(struct gensio *io, int err, void *open_data)
     assert(!od->finished);
     OOMLOCK(&od->lock);
     assert(!id->open_done);
-    o->wake(od->waiter);
+    gensio_os_funcs_wake(o, od->waiter);
     if (id->closed)
 	goto out_unlock;
 
@@ -1360,13 +1402,13 @@ alloc_od(struct oom_tests *test)
 {
     struct oom_test_data *od;
 
-    od = o->zalloc(o, sizeof(*od));
+    od = gensio_os_funcs_zalloc(o, sizeof(*od));
     if (!od)
 	return NULL;
     od->refcount = 1;
-    od->waiter = o->alloc_waiter(o);
+    od->waiter = gensio_os_funcs_alloc_waiter(o);
     if (!od->waiter) {
-	o->free(o, od);
+	gensio_os_funcs_zfree(o, od);
 	return NULL;
     }
     od->ccon.od = od;
@@ -1390,7 +1432,8 @@ wait_for_data(struct oom_test_data *od, gensio_time *timeout)
 
     for (;;) {
 	OOMUNLOCK(&od->lock);
-	rv = o->wait_intr_sigmask(od->waiter, 1, timeout, proc_data);
+	rv = gensio_os_funcs_wait_intr_sigmask(o, od->waiter,
+					       1, timeout, proc_data);
 	OOMLOCK(&od->lock);
 	if (rv == GE_INTERRUPTED)
 	    continue;
@@ -1398,7 +1441,7 @@ wait_for_data(struct oom_test_data *od, gensio_time *timeout)
 		      od->ccon.err == OOME_READ_OVERFLOW) {
 	    printf("Waiting on err A: %s\n", gensio_err_to_str(rv));
 	    fflush(stdout);
-	    handle_timeout_err();
+	    handle_timeout_err(od);
 	}
 	if (rv) {
 	    err = rv;
@@ -1436,12 +1479,13 @@ close_con(struct io_test_data *id, gensio_time *timeout)
     /* Make sure the open completes. */
     while (!id->open_done) {
 	OOMUNLOCK(&od->lock);
-	rv = o->wait_intr_sigmask(id->od->waiter, 1, timeout, proc_data);
+	rv = gensio_os_funcs_wait_intr_sigmask(o, id->od->waiter,
+					       1, timeout, proc_data);
 	OOMLOCK(&od->lock);
 	if (rv == GE_TIMEDOUT) {
 	    printf("Waiting on timeout err A\n");
 	    fflush(stdout);
-	    handle_timeout_err();
+	    handle_timeout_err(od);
 	}
 	if (rv == GE_INTERRUPTED) {
 	    rv = 0;
@@ -1475,12 +1519,13 @@ close_stderr(struct oom_test_data *od, gensio_time *timeout)
 
     while (!od->stderr_rem_closed) {
 	OOMUNLOCK(&od->lock);
-	rv = o->wait_intr_sigmask(od->waiter, 1, timeout, proc_data);
+	rv = gensio_os_funcs_wait_intr_sigmask(o, od->waiter,
+					       1, timeout, proc_data);
 	OOMLOCK(&od->lock);
 	if (rv == GE_TIMEDOUT) {
 	    printf("Waiting on timeout err G1\n");
 	    fflush(stdout);
-	    handle_timeout_err();
+	    handle_timeout_err(od);
 	}
 	if (rv == GE_INTERRUPTED)
 	    continue;
@@ -1500,12 +1545,13 @@ close_stderr(struct oom_test_data *od, gensio_time *timeout)
     od_ref(od); /* Ref for the close */
     while (od->ccon_stderr_io) {
 	OOMUNLOCK(&od->lock);
-	rv = o->wait_intr_sigmask(od->waiter, 1, timeout, proc_data);
+	rv = gensio_os_funcs_wait_intr_sigmask(o, od->waiter,
+					       1, timeout, proc_data);
 	OOMLOCK(&od->lock);
 	if (rv == GE_TIMEDOUT) {
 	    printf("Waiting on timeout err G\n");
 	    fflush(stdout);
-	    handle_timeout_err();
+	    handle_timeout_err(od);
 	}
 	if (rv == GE_INTERRUPTED)
 	    continue;
@@ -1535,12 +1581,13 @@ close_cons(struct oom_test_data *od, bool close_acc, gensio_time *timeout)
 
     while (!err && (od->ccon.io || od->scon.io)) {
 	OOMUNLOCK(&od->lock);
-	rv = o->wait_intr_sigmask(od->waiter, 1, timeout, proc_data);
+	rv = gensio_os_funcs_wait_intr_sigmask(o, od->waiter,
+					       1, timeout, proc_data);
 	OOMLOCK(&od->lock);
 	if (rv == GE_TIMEDOUT) {
 	    printf("Waiting on timeout err B\n");
 	    fflush(stdout);
-	    handle_timeout_err();
+	    handle_timeout_err(od);
 	}
 	if (rv == GE_INTERRUPTED)
 	    continue;
@@ -1631,7 +1678,7 @@ run_oom_test(struct oom_tests *test, long count, int *exitcode,
 
     rv = str_to_gensio(constr, o, con_cb, &od->ccon, &od->ccon.io);
     assert(!debug || !rv);
-    o->free(o, constr);
+    gensio_os_funcs_zfree(o, constr);
     if (rv)
 	goto out_err;
 
@@ -1663,12 +1710,13 @@ run_oom_test(struct oom_tests *test, long count, int *exitcode,
 	    od_ref(od); /* Ref for the close */
 	    while (od->acc) {
 		OOMUNLOCK(&od->lock);
-		rv = o->wait_intr_sigmask(od->waiter, 1, &timeout, proc_data);
+		rv = gensio_os_funcs_wait_intr_sigmask(o, od->waiter,
+						       1, &timeout, proc_data);
 		OOMLOCK(&od->lock);
 		if (rv == GE_TIMEDOUT) {
 		    printf("Waiting on timeout err C\n");
 		    fflush(stdout);
-		    handle_timeout_err();
+		    handle_timeout_err(od);
 		}
 		if (rv == GE_INTERRUPTED)
 		    continue;
@@ -1748,7 +1796,7 @@ run_oom_acc_test(struct oom_tests *test, long count, int *exitcode,
 
     err = str_to_gensio(constr, o, con_cb, &od->ccon, &od->ccon.io);
     assert(!debug || !err);
-    o->free(o, constr);
+    gensio_os_funcs_zfree(o, constr);
     if (err)
 	goto out_err;
 
@@ -1763,7 +1811,8 @@ run_oom_acc_test(struct oom_tests *test, long count, int *exitcode,
 
     for (;;) {
 	OOMUNLOCK(&od->lock);
-	rv = o->wait_intr_sigmask(od->waiter, 1, &timeout, proc_data);
+	rv = gensio_os_funcs_wait_intr_sigmask(o, od->waiter,
+					       1, &timeout, proc_data);
 	OOMLOCK(&od->lock);
 	if (debug && rv == GE_TIMEDOUT) {
 	    printf("Waiting on err E\n");
@@ -1804,7 +1853,7 @@ run_oom_acc_test(struct oom_tests *test, long count, int *exitcode,
 
     err = str_to_gensio(locstr, o, con_cb, &od->scon, &od->scon.io);
     assert(!debug || !err);
-    o->free(o, locstr);
+    gensio_os_funcs_zfree(o, locstr);
     if (err)
 	goto out_err;
 
@@ -2101,6 +2150,10 @@ main(int argc, char *argv[])
 	    gensio_set_log_mask(GENSIO_LOG_MASK_ALL);
 	} else if (strcmp(argv[i], "-b") == 0) {
 	    sleep_on_timeout_err = true;
+#ifndef _WIN32
+	} else if (strcmp(argv[i], "-k") == 0) {
+	    kill_on_timeout_err = true;
+#endif
 	} else if (strcmp(argv[i], "-l") == 0) {
 	    list_tests = true;
 	} else if (strcmp(argv[i], "-t") == 0) {
@@ -2178,7 +2231,7 @@ main(int argc, char *argv[])
 
     if (use_glib) {
 #ifndef HAVE_GLIB
-	fprintf(stderr, "glib specified, but glib OS handler not avaiable.\n");
+	fprintf(stderr, "glib specified, but glib OS handler not available.\n");
 	exit(1);
 #else
 	os_func_str = " --glib";
@@ -2186,7 +2239,7 @@ main(int argc, char *argv[])
 #endif
     } else if (use_tcl) {
 #ifndef HAVE_TCL
-	fprintf(stderr, "tcl specified, but tcl OS handler not avaiable.\n");
+	fprintf(stderr, "tcl specified, but tcl OS handler not available.\n");
 	exit(1);
 #else
 	if (num_extra_threads > 0)
@@ -2204,7 +2257,7 @@ main(int argc, char *argv[])
 		gensio_err_to_str(rv));
 	goto out_err;
     }
-    o->vlog = do_vlog;
+    gensio_os_funcs_set_vlog(o, do_vlog);
 
     rv = gensio_os_proc_setup(o, &proc_data);
     if (rv) {
@@ -2250,10 +2303,10 @@ main(int argc, char *argv[])
 	exit(1);
     }
     if (i >= argc)
-	o->free(o, s);
+	gensio_os_funcs_zfree(o, s);
 
     for (i = 0; i < num_extra_threads; i++) {
-	loopwaiter[i] = o->alloc_waiter(o);
+	loopwaiter[i] = gensio_os_funcs_alloc_waiter(o);
 	if (!loopwaiter[i]) {
 	    fprintf(stderr, "Could not allocate loop waiter\n");
 	    goto out_err;
@@ -2296,18 +2349,18 @@ main(int argc, char *argv[])
     gensio_argv_free(o, env.argv);
 
     for (i = 0; i < num_extra_threads; i++) {
-	o->wake(loopwaiter[i]);
+	gensio_os_funcs_wake(o, loopwaiter[i]);
 	gensio_os_wait_thread(loopth[i]);
-	o->free_waiter(loopwaiter[i]);
+	gensio_os_funcs_free_waiter(o, loopwaiter[i]);
     }
 
     for (i = 0; oom_tests[i].connecter; i++) {
 	if (oom_tests[i].free_connecter)
-	    o->free(o, oom_tests[i].connecter);
+	    gensio_os_funcs_zfree(o, oom_tests[i].connecter);
     }
 
     printf("Got %ld errors, skipped %ld tests\n", errcount, skipcount);
-    while (o && o->service(o, &zerotime) == 0)
+    while (o && gensio_os_funcs_service(o, &zerotime) == 0)
 	;
     cleanup_ods();
     gensio_cleanup_mem(o);
@@ -2317,10 +2370,10 @@ main(int argc, char *argv[])
 
  out_err:
     gensio_os_proc_cleanup(proc_data);
-    o->free(o, gensiot);
+    gensio_os_funcs_zfree(o, gensiot);
     cleanup_ods();
     gensio_cleanup_mem(o);
-    o->free_funcs(o);
+    gensio_os_funcs_free(o);
     gensio_osfunc_exit(1);
     return 1;
 }
