@@ -53,6 +53,7 @@
 #include "utils.h"
 
 unsigned int debug;
+struct gensio_os_proc_data *proc_data;
 
 #if HAVE_OPENSSL
 /*
@@ -285,7 +286,7 @@ io_closed(struct gensio *io, void *close_data)
 }
 
 static void
-i_gshutdown(struct ioinfo *ioinfo, bool user_req)
+i_gshutdown(struct ioinfo *ioinfo, enum ioinfo_shutdown_reason reason)
 {
     struct ioinfo *oioinfo = ioinfo_otherioinfo(ioinfo);
     struct gtconn_info *gtconn = ioinfo_userdata(ioinfo);
@@ -313,18 +314,18 @@ i_gshutdown(struct ioinfo *ioinfo, bool user_req)
 	if (err)
 	    i_io_closed(ogtconn->close_io, NULL);
     }
-    if (!g->err && !user_req)
+    if (!g->err && reason == IOINFO_SHUTDOWN_ERR)
 	g->err = GE_IOERR;
 }
 
 static void
-gshutdown(struct ioinfo *ioinfo, bool user_req)
+gshutdown(struct ioinfo *ioinfo, enum ioinfo_shutdown_reason reason)
 {
     struct gtconn_info *gtconn = ioinfo_userdata(ioinfo);
     struct gtinfo *g = gtconn->g;
 
     gensio_os_funcs_lock(g->o, g->lock);
-    i_gshutdown(ioinfo, user_req);
+    i_gshutdown(ioinfo, reason);
     gensio_os_funcs_unlock(g->o, g->lock);
 }
 
@@ -435,6 +436,45 @@ print_io_addr(struct gensio *io, bool local)
     return 0;
 }
 
+static void
+winch_ready(int x_chrs, int y_chrs, int x_bits, int y_bits,
+	    void *handler_data)
+{
+    struct ioinfo *ioinfo = handler_data;
+    struct ioinfo *oioinfo = ioinfo_otherioinfo(ioinfo);
+    struct gensio *oio = ioinfo_io(oioinfo);
+    char *str;
+
+    str = alloc_sprintf("%d:%d:%d:%d", y_chrs, x_chrs, x_bits, y_bits);
+    if (!str)
+	return;
+    gensio_control(oio, GENSIO_CONTROL_DEPTH_FIRST,
+		   GENSIO_CONTROL_SET, GENSIO_CONTROL_WIN_SIZE,
+		   str, 0);
+    free(str);
+}
+
+static void
+reg_winch(struct ioinfo *ioinfo)
+{
+    struct gensio *io = ioinfo_io(ioinfo);
+    struct gensio_iod *iod;
+    gensiods len = sizeof(iod);
+    int err;
+
+    /*
+     * Which iod is passed in the data as an index, but the iod is
+     * returned in the same data.  It looks a little strange to do
+     * this, but that's how it works.
+     */
+    memcpy(&iod, "0", 2);
+    err = gensio_control(io, GENSIO_CONTROL_DEPTH_FIRST, true,
+			 GENSIO_CONTROL_IOD, (char *) &iod, &len);
+    if (!err)
+	gensio_os_proc_register_winsize_handler(proc_data, iod,
+						winch_ready, ioinfo);
+}
+
 static void
 io_open(struct gensio *io, int err, void *open_data)
 {
@@ -447,9 +487,10 @@ io_open(struct gensio *io, int err, void *open_data)
 	    g->err = err;
 	report_err(gtconn->g, "open error on %s: %s", gtconn->ios,
 		   gensio_err_to_str(err));
-	gshutdown(ioinfo, false);
+	gshutdown(ioinfo, IOINFO_SHUTDOWN_ERR);
     } else {
 	ioinfo_set_ready(ioinfo, io);
+	reg_winch(ioinfo);
     }
 }
 
@@ -468,7 +509,7 @@ io_open_paddr(struct gensio *io, int err, void *open_data)
 	    g->err = err;
 	report_err(g, "open error on %s: %s", gtconn->ios,
 		   gensio_err_to_str(err));
-	gshutdown(ioinfo, false);
+	gshutdown(ioinfo, IOINFO_SHUTDOWN_ERR);
     } else {
 	if (g->print_laddr)
 	    print_io_addr(io, true);
@@ -479,9 +520,10 @@ io_open_paddr(struct gensio *io, int err, void *open_data)
 	if (rv) {
 	    report_err(g, "Could not open %s: %s", ogtconn->ios,
 		       gensio_err_to_str(rv));
-	    gshutdown(ioinfo, false);
+	    gshutdown(ioinfo, IOINFO_SHUTDOWN_ERR);
 	} else {
 	    ioinfo_set_ready(ioinfo, io);
+	    reg_winch(ioinfo);
 	}
     }
 }
@@ -549,6 +591,7 @@ add_io(struct gtinfo *g, struct gensio *io, bool open_finished)
 
     if (open_finished) {
 	ioinfo_set_ready(ioinfo2, gtconn2->io);
+	reg_winch(ioinfo2);
 	if (g->print_laddr)
 	    print_io_addr(io, true);
 	if (g->print_raddr)
@@ -666,7 +709,7 @@ io_acc_event(struct gensio_accepter *accepter, void *user_data,
 		g->err = err;
 		report_err(g, "Could not open %s: %s", ogtconn->ios,
 			gensio_err_to_str(err));
-		i_gshutdown(ioinfo, false);
+		i_gshutdown(ioinfo, IOINFO_SHUTDOWN_ERR);
 		goto out_unlock;
 	    }
 	} else {
@@ -791,7 +834,6 @@ main(int argc, char *argv[])
     bool use_tcl = false;
     gensio_time endwait = { 5, 0 };
     struct gensio *io = NULL;
-    struct gensio_os_proc_data *proc_data = NULL;
     unsigned int num_extra_threads = 0, i;
     struct gensio_loop_info *loopinfo = NULL;
 
@@ -1012,7 +1054,7 @@ main(int argc, char *argv[])
 
 	    fprintf(stderr, "Could not open %s: %s\n", g.ios2,
 		    gensio_err_to_str(rv));
-	    i_gshutdown(ioinfo, false);
+	    i_gshutdown(ioinfo, IOINFO_SHUTDOWN_ERR);
 	}
 	gensio_os_funcs_unlock(g.o, g.lock);
 	io = NULL;
