@@ -23,6 +23,7 @@
  *  release a modified version which carries forward this exception.
  */
 
+#include "config.h"
 #include <stdio.h>
 #include <stdbool.h>
 #include <sys/types.h>
@@ -40,6 +41,8 @@
 #include "gtlssh.h"
 
 int debug;
+struct gensio_os_funcs *o;
+struct gensio_os_proc_data *proc_data;
 
 #define DEFAULT_RSA_KEYSIZE 2048
 #define DEFAULT_EC_KEYSIZE 256
@@ -67,6 +70,20 @@ static bool commonname_set = false;
 
 static bool force = false;
 
+static void
+glogger(void *cbdata, const char *format, ...)
+{
+    va_list va;
+
+    va_start(va, format);
+    vfprintf(stderr, format, va);
+    va_end(va);
+}
+
+#ifdef _WIN32
+static int storepw(void);
+#endif
+
 static void
 help(const char *progname)
 {
@@ -77,12 +94,13 @@ help(const char *progname)
     P("  --keysize <size> - Create an RSA key with the given number of bits.\n");
     P("        default is %u for rsa, %u for ec\n", DEFAULT_RSA_KEYSIZE,
       DEFAULT_EC_KEYSIZE);
+    P("        Do not specify for ed25519\n");
     P("  --keydays <days> - Create a key that expires in the given number\n");
     P("        of days.  Default is %u\n", DEFAULT_KEYDAYS);
     P("  --basedir <dir> - Location where keys are stored.\n");
     P("        Default is %s\n", default_gtlsshdir);
     P("  --keydir <dir> - Location to put the non-default generated keys.\n");
-    P("        Default is %s for normal certificates.\n", keydir);
+    P("        Default is %s for normal certificates.\n", default_keydir);
 #ifdef DEFAULT_CONFDIR
     P("        %s for server certificates.\n", DEFAULT_CONFDIR);
 #endif
@@ -90,7 +108,7 @@ help(const char *progname)
     P("        The default is your username for normal certificates and\n");
     P("        the fully qualified domain name for server certificates.\n");
     P("  --algorithm <algname> - Set the algorithm to use for the key,\n");
-    P("        either rsa or ec.  The default is rsa\n");
+    P("        either ed25519, rsa or ec.  The default is ed25519.\n");
     P("  --force, -f - Don't ask questions, just do the operation.  This\n");
     P("        may overwrite data without asking.\n");
     P("  --version - Print the version number and exit.\n");
@@ -104,11 +122,11 @@ help(const char *progname)
     P("    Generate a keys for the given hostnames, optionally at the given\n");
     P("    port.  If no hostname is given, the default key/cert is generated\n");
     P("    in\n");
-    P("      %s/default.key\n", default_gtlsshdir);
+    P("      %s%cdefault.key\n", default_gtlsshdir, DIRSEP);
     P("    and\n");
-    P("      %s/default.crt\n", default_gtlsshdir);
+    P("      %s%cdefault.crt\n", default_gtlsshdir, DIRSEP);
     P("    Otherwise the key is generated in\n");
-    P("      %s/<hostname>[,<port>].key/crt.\n", default_keydir);
+    P("      %s%c<hostname>[,<port>].key/crt.\n", default_keydir, DIRSEP);
     P("    When gtlssh makes a connection, it will look for the hostname\n");
     P("    with the port, then just the hostname, then the default key in\n");
     P("    that order for the key to use for the connection.\n");
@@ -120,14 +138,14 @@ help(const char *progname)
     P("    certificates into those directories but do not rehash them,\n");
     P("    the tools will not be able to find the new certificates.\n");
     P("    If you don't enter any directories, it will rehash the following:\n");
-    P("      %s/allowed_certs\n", default_gtlsshdir);
-    P("      %s/server_certs\n", default_gtlsshdir);
+    P("      %s%callowed_certs\n", default_gtlsshdir, DIRSEP);
+    P("      %s%cserver_certs\n", default_gtlsshdir, DIRSEP);
     P("    Certificates that have expired are automatically removed.\n");
     P("\n");
     P("  addallow [-i] <hostname> <file>\n");
     P("    Add the given file as an allowed public certificate for the given\n");
     P("    hostname.  It will install this file in the directory:\n");
-    P("      %s/allowed_certs\n", default_gtlsshdir);
+    P("      %s%callowed_certs\n", default_gtlsshdir, DIRSEP);
     P("    with the name 'hostname.crt'.  It will also rehash the\n");
     P("    directory.  If -i is specified, input comes from stdin and the\n");
     P("    file is not required or used.  If the destination file already\n");
@@ -149,6 +167,15 @@ help(const char *progname)
     P("    manually.  By default the credential on the remote host is\n");
     P("    named the output of 'hostname -f' on the local machine,\n");
     P("    -n overrides this.\n");
+#ifdef _WIN32
+    P("\n");
+    P("  storepw\n");
+    P("    On Windows, logons do not work very well without credentials,\n");
+    P("    but gtlsshd doesn't have a way to get them on a certificate.\n");
+    P("    logon.  This lets you store your password in your .gtlssh\n");
+    P("    directory so gtlsshd can pick it up.  Not ideal, but it's\n");
+    P("    not terrible, as the private keys are there, anyway.\n");
+#endif
 #ifdef DEFAULT_CONFDIR
     P("\n");
     P("  serverkey [name]\n");
@@ -227,12 +254,12 @@ check_dir(bool created, bool check_private, const char *fmt, ...)
 	exit(1);
     }
 
-    if (!check_dir_exists(dir, check_private)) {
+    if (!check_dir_exists(glogger, NULL, dir, check_private)) {
 	if (created) {
-	    make_dir(dir, check_private);
+	    make_dir(glogger, NULL, dir, check_private);
 	} else {
 	    if (force || promptyn("%s does not exist.  Create it?", dir)) {
-		make_dir(dir, check_private);
+		make_dir(glogger, NULL, dir, check_private);
 	    } else {
 		printf("Not creating %s, giving up\n", dir);
 		exit(1);
@@ -247,10 +274,10 @@ check_dirstruct(void)
 {
     bool created = false;
 
-    if (!check_dir_exists(gtlsshdir, true)) {
+    if (!check_dir_exists(glogger, NULL, gtlsshdir, true)) {
 	if (force || promptyn("%s does not exist.  Do you want to create it?",
 			      gtlsshdir)) {
-	    make_dir(gtlsshdir, true);
+	    make_dir(glogger, NULL, gtlsshdir, true);
 	    created = true;
 	} else {
 	    printf("Not modifying %s, giving up\n", gtlsshdir);
@@ -300,7 +327,7 @@ check_for_certfile(const char *dir, const char *name, void *cbdata)
 {
     const char *s = strrchr(name, '.');
 
-    if (strcmp(s, ".crt") == 0) {
+    if (!s || strcmp(s, ".crt") == 0) {
 	bool *certpresent = cbdata;
 
 	*certpresent = true;
@@ -342,7 +369,7 @@ remove_links(const char *dir, const char *name, void *cbdata)
 		dir, DIRSEP, name);
 	return ITERATE_DIR_ERR;
     }
-    rv = delete_file(fname);
+    rv = delete_file(glogger, NULL, fname);
     free(fname);
     if (rv) {
 	fprintf(stderr, "Unable to remove %s%c%s: %s", dir, DIRSEP, name,
@@ -458,7 +485,7 @@ hash_file(const char *dir, const char *name, void *cbdata)
     const ASN1_TIME *notafter;
     const char *endstr = strrchr(name, '.');
 
-    if (strcmp(endstr, ".crt") != 0)
+    if (!endstr || (strcmp(endstr, ".crt") != 0))
 	return ITERATE_DIR_CONTINUE;
 
     s = alloc_sprintf("%s%c%s", dir, DIRSEP, name);
@@ -474,7 +501,7 @@ hash_file(const char *dir, const char *name, void *cbdata)
     notafter = X509_get0_notAfter(cert);
     if (X509_cmp_current_time(notafter) < 0) {
 	printf("Removing expired certificate %s\n", s);
-	delete_file(s);
+	delete_file(glogger, NULL, s);
 	free(s);
 	free_cert(cert);
 	return ITERATE_DIR_CONTINUE;
@@ -504,7 +531,7 @@ hash_file(const char *dir, const char *name, void *cbdata)
 	    goto out_err;
 	}
 
-	err = make_link(l, s, name);
+	err = make_link(glogger, NULL, l, s, name);
 	free(l);
 	if (err == 0)
 	    goto out;
@@ -582,9 +609,7 @@ rehash(int argc, char *argv[])
 	hash_dir("%s%cserver_certs", gtlsshdir, DIRSEP);
     } else {
 	for (i = 0; i < argc; i++) {
-	    if (!check_dir_exists(argv[i], false))
-		fprintf(stderr, "%s is not a directory", argv[i]);
-	    else
+	    if (check_dir_exists(glogger, NULL, argv[i], false))
 		hash_dir("%s", argv[i]);
 	}
     }
@@ -630,14 +655,15 @@ addallow(int argc, char **argv)
     dest = alloc_sprintf("%s%callowed_certs%c%s.crt", gtlsshdir, DIRSEP,
 			 DIRSEP, argv[0]);
     if (!dest) {
-	fprintf(stderr, "Can't allocate memory for %s/allowed_certs/%s.crt\n",
-		gtlsshdir, argv[0]);
+	fprintf(stderr,
+		"Can't allocate memory for %s%callowed_certs%c%s.crt\n",
+		gtlsshdir, DIRSEP, DIRSEP, argv[0]);
 	return 1;
     }
 
     if (check_file_exists(dest)) {
 	if (force || promptyn("File %s already exists, do you want to overwrite it?", dest)) {
-	    delete_file(dest);
+	    delete_file(glogger, NULL, dest);
 	} else {
 	    printf("Not installing certificate at %s\n", dest);
 	    goto out_err;
@@ -767,7 +793,45 @@ genpkey_ec(const char *key)
     return rc != 0;
 }
 
-static int (*genpkey)(const char *key) = genpkey_rsa;
+/*
+ * Generate the given private ED25519 key.
+ */
+static int
+genpkey_ed25519(const char *key)
+{
+    const char *argv[7];
+    char *out, *errout;
+    int err, rc;
+
+    if (keysize_set) {
+	fprintf(stderr, "Keysize not accepted for ED25519\n");
+	return 1;
+    }
+    argv[0] = "openssl";
+    argv[1] = "genpkey";
+    argv[2] = "-algorithm";
+    argv[3] = "ed25519";
+    argv[4] = "-out";
+    argv[5] = key;
+    argv[6] = NULL;
+
+    err = run_get_output(argv, true, NULL, 0,
+			 NULL, 0, &out, NULL, &errout, NULL, &rc);
+    if (err)
+	return 1;
+
+    if (rc) {
+	fprintf(stderr, "Error running openssl: %s\n", errout);
+    } else {
+	printf("Key created at %s.\n", key);
+    }
+
+    free(out);
+    free(errout);
+    return rc != 0;
+}
+
+static int (*genpkey)(const char *key) = genpkey_ed25519;
 
 /*
  * Create a single key.  If name is NULL, it's a server key, otherwise
@@ -785,16 +849,16 @@ keygen_one(const char *name, const char *key, const char *cert)
 	    /* Move the key and certificate to backup files. */
 	    s = alloc_sprintf("%s.1", key);
 	    if (s) {
-		move_file(key, s);
+		move_file(glogger, NULL, key, s);
 		free(s);
 	    }
 	    s = alloc_sprintf("%s.1", cert);
 	    if (s) {
-		move_file(cert, s);
+		move_file(glogger, NULL, cert, s);
 		free(s);
 	    }
-	    delete_file(key);
-	    delete_file(cert);
+	    delete_file(glogger, NULL, key);
+	    delete_file(glogger, NULL, cert);
 	} else if (name) {
 	    printf("Not generating key for %s", name);
 	    return 1;
@@ -845,7 +909,7 @@ keygen_one(const char *name, const char *key, const char *cert)
     } else {
 	if (name) {
 	    printf("Certificate created.  Put %s into:\n", cert);
-	    printf("  .gtlssh/allowed_certs\n");
+	    printf("  .gtlssh%callowed_certs\n", DIRSEP);
 	    printf("on remote systems you want to log into"
 		   " without a password.\n");
 	} else {
@@ -879,7 +943,7 @@ keygen(int argc, char *argv[])
 	err = keygen_one("default", key, cert);
 	free(key);
 	free(cert);
-	return err;
+	goto out_finish;
     }
 
     port[0] = '\0';
@@ -914,6 +978,19 @@ keygen(int argc, char *argv[])
 	}
     }
     return 0;
+
+ out_finish:
+#ifdef _WIN32
+    printf("\n");
+    printf("On Windows, logons do not work very well without credentials,\n");
+    printf("but gtlsshd doesn't have a way to get them on a certificate.\n");
+    printf("logon.  This lets you store your password in your .gtlssh\n");
+    printf("directory so gtlsshd can pick it up.  Not ideal, but it's\n");
+    printf("not terrible, as the private keys are there, anyway.\n");
+    if (promptyn("Do you want to store your password?"))
+	storepw();
+#endif
+    return err;
 }
 
 #ifdef DEFAULT_CONFDIR
@@ -940,8 +1017,8 @@ serverkey(int inargc, char *inargv[])
 	goto out;
     }
 
-    if (!check_dir_exists(confdir, false))
-	make_dir(confdir, false);
+    if (!check_dir_exists(glogger, NULL, confdir, false))
+	make_dir(glogger, NULL, confdir, false);
 
     rv = keygen_one(NULL, serverkey, servercert);
 
@@ -1073,7 +1150,7 @@ pushcert(int argc, char *argv[])
 	return 1;
     }
 
-    hostname = get_my_hostname();
+    hostname = get_my_hostname(glogger, NULL);
     if (!hostname) {
 	fprintf(stderr, "Out of memory allocating hostname\n");
 	return 1;
@@ -1129,13 +1206,98 @@ pushcert(int argc, char *argv[])
     return rv;
 }
 
+#ifdef _WIN32
+static int
+storepw(void)
+{
+    struct gensio *tty = NULL;
+    int err, rv = 1;
+    const char *constr = "stdio(console,raw)";
+    const char *prompt = "Password: ";
+    char password[100];
+    size_t pos = 0;
+    char *pwfile = alloc_sprintf("%s%cpassword", gtlsshdir, DIRSEP);
+
+    if (!pwfile) {
+	fprintf(stderr, "Error allocating password filename: Mo memory\n");
+	return 1;
+    }
+
+    err = str_to_gensio(constr, o, NULL, NULL, &tty);
+    if (err) {
+	fprintf(stderr, "Error allocating gensio: %s\n",
+		gensio_err_to_str(err));
+	goto out_err;
+    }
+
+    err = gensio_open_s(tty);
+    if (err) {
+	gensio_free(tty);
+	tty = NULL;
+	fprintf(stderr, "Unable to open console %s: %s\n", constr,
+		gensio_err_to_str(err));
+	goto out_err;
+    }
+
+    err = gensio_set_sync(tty);
+    if (err) {
+	fprintf(stderr, "Unable to set console sync %s: %s\n", constr,
+		gensio_err_to_str(err));
+	goto out_err;
+    }
+
+    err = gensio_write_s(tty, NULL, prompt, strlen(prompt), NULL);
+    if (err) {
+	fprintf(stderr, "Error writing password prompt, giving up: %s\n",
+		gensio_err_to_str(err));
+	goto out_err;
+    }
+
+    while (true) {
+	gensiods count = 0;
+	char c;
+
+	err = gensio_read_s(tty, &count, &c, 1, NULL);
+	if (err) {
+	    fprintf(stderr, "Error reading password: %s\n",
+		    gensio_err_to_str(err));
+	    goto out_err;
+	}
+	if (count != 1) {
+	    fprintf(stderr,
+		    "Error reading password: read didn't return data\n");
+	    err = GE_IOERR;
+	    goto out_err;
+	}
+	if (c == '\r' || c == '\n')
+	    break;
+	if (pos < sizeof(password) - 1)
+	    password[pos++] = c;
+    }
+    gensio_write_s(tty, NULL, "\r\n", 2, NULL);
+    password[pos++] = '\0';
+
+    rv = make_file(glogger, NULL, pwfile, password, pos, true);
+
+ out_err:
+    memset(password, 0, sizeof(password));
+    if (pwfile)
+	free(pwfile);
+    if (tty) {
+	gensio_close_s(tty);
+	gensio_free(tty);
+    }
+    return rv;
+}
+#endif
+
 int
 main(int argc, char **argv)
 {
-    int i, rv = 1;
+    int i, rv = 1, err;
     const char *algorithm = NULL;
 
-    default_gtlsshdir = get_tlsshdir();
+    default_gtlsshdir = get_tlsshdir(glogger, NULL, NULL, NULL);
     if (!default_gtlsshdir)
 	exit(1);
     default_keydir = alloc_sprintf("%s%ckeycerts", default_gtlsshdir, DIRSEP);
@@ -1195,6 +1357,8 @@ main(int argc, char **argv)
 	    genpkey = genpkey_rsa;
 	} else if (strcmp(algorithm, "ec") == 0) {
 	    genpkey = genpkey_ec;
+	} else if (strcmp(algorithm, "ed25519") == 0) {
+	    genpkey = genpkey_ed25519;
 	} else {
 	    fprintf(stderr, "Invalid algorithm: %s\n", algorithm);
 	    exit(1);
@@ -1213,6 +1377,20 @@ main(int argc, char **argv)
 	exit(1);
     }
 
+    err = gensio_default_os_hnd(0, &o);
+    if (err) {
+	fprintf(stderr, "Error allocating os handler: %s\n",
+		gensio_err_to_str(err));
+	exit(1);
+    }
+
+    err = gensio_os_proc_setup(o, &proc_data);
+    if (err) {
+	fprintf(stderr, "Could not setup process data: %s\n",
+		gensio_err_to_str(err));
+	exit(1);
+    }
+
     if (strcmp(argv[0], "rehash") == 0)
 	rv = rehash(argc - 1, argv + 1);
     else if (strcmp(argv[0], "addallow") == 0)
@@ -1221,7 +1399,7 @@ main(int argc, char **argv)
 	if (!keydir_set)
 	    check_dirstruct();
 	if (!commonname_set) {
-	    alloc_commonname = get_my_username();
+	    alloc_commonname = get_my_username(glogger, NULL);
 	    if (!alloc_commonname) {
 		fprintf(stderr, "Error allocating username\n");
 		exit(1);
@@ -1234,7 +1412,7 @@ main(int argc, char **argv)
     } else if (strcmp(argv[0], "setup") == 0) {
 	check_dirstruct();
 	if (!commonname_set) {
-	    alloc_commonname = get_my_username();
+	    alloc_commonname = get_my_username(glogger, NULL);
 	    if (!alloc_commonname) {
 		fprintf(stderr, "Error allocating username\n");
 		exit(1);
@@ -1249,7 +1427,7 @@ main(int argc, char **argv)
 	if (keydir_set)
 	    confdir = keydir;
 	if (!commonname_set) {
-	    alloc_commonname = get_my_hostname();
+	    alloc_commonname = get_my_hostname(glogger, NULL);
 	    if (!alloc_commonname) {
 		fprintf(stderr, "Error allocating hostname\n");
 		exit(1);
@@ -1262,10 +1440,19 @@ main(int argc, char **argv)
 #endif
     } else if (strcmp(argv[0], "pushcert") == 0) {
 	rv = pushcert(argc - 1, argv + 1);
+#ifdef _WIN32
+    } else if (strcmp(argv[0], "storepw") == 0) {
+	rv = storepw();
+#endif
     } else {
 	fprintf(stderr, "Unknown command %s, use --help for help\n", argv[0]);
 	exit(1);
     }
 
+    if (proc_data)
+	gensio_os_proc_cleanup(proc_data);
+    if (o)
+	gensio_os_funcs_free(o);
+
     return rv;
 }
