@@ -63,7 +63,17 @@ struct gdata {
 struct gensio_os_proc_data *proc_data;
 
 static void
-gshutdown(struct ioinfo *ioinfo, bool user_req)
+glogger(void *cbdata, const char *format, ...)
+{
+    va_list va;
+
+    va_start(va, format);
+    vfprintf(stderr, format, va);
+    va_end(va);
+}
+
+static void
+gshutdown(struct ioinfo *ioinfo, enum ioinfo_shutdown_reason reason)
 {
     struct gdata *ginfo = ioinfo_userdata(ioinfo);
 
@@ -137,7 +147,8 @@ goobdata(struct ioinfo *ioinfo, unsigned char *buf, gensiods *buflen)
 	/*
 	 * Which iod is passed in the data as an index, but the iod is
 	 * returned in the same data.  It looks a little strange to do
-	 * this, but that's how it works.
+	 * this, but that's how it works.  We want stdin, which is the
+	 * first one.
 	 */
 	memcpy(&iod, "0", 2);
 	err = gensio_control(io, GENSIO_CONTROL_DEPTH_FIRST, true,
@@ -724,15 +735,15 @@ lookup_certinfo(struct gensio_os_funcs *o,
 	keyname = tkeyname;
     }
 
-    err = checkout_file(CAname, true, false);
+    err = checkout_file(glogger, NULL, CAname, true, false);
     if (err)
 	goto out_err;
 
-    err = checkout_file(certname, false, false);
+    err = checkout_file(glogger, NULL, certname, false, false);
     if (err)
 	goto out_err;
 
-    err = checkout_file(keyname, false, true);
+    err = checkout_file(glogger, NULL, keyname, false, true);
     if (err)
 	goto out_err;
 
@@ -1281,7 +1292,7 @@ winch_ready(int x_chrs, int y_chrs, int x_bits, int y_bits,
 }
 
 static void
-pr_localport(const char *fmt, va_list ap)
+pr_localport(void *cb_data, const char *fmt, va_list ap)
 {
     vfprintf(stderr, fmt, ap);
 }
@@ -1302,6 +1313,7 @@ struct remote_portinfo {
 
 static unsigned int curr_service;
 static struct remote_portinfo *remote_ports;
+static struct local_ports *locport;
 
 static void
 handle_rem_req(struct gensio *io, const char *service)
@@ -1318,7 +1330,7 @@ handle_rem_req(struct gensio *io, const char *service)
 	return;
     }
 
-    remote_port_new_con(pi->o, io, pi->connecter_str, pi->id_str);
+    remote_port_new_con(locport, io, pi->connecter_str, pi->id_str);
 }
 
 static int
@@ -1484,19 +1496,30 @@ handle_port(struct gensio_os_funcs *o, bool remote, const char *iaddr)
 	accepter_str = alloc_sprintf("%s,%s", type, s[pos]);
 	pos++;
     }
+    if (!accepter_str) {
+	fprintf(stderr, "Unable to allocate accepter string");
+	err = GE_NOMEM;
+	goto out_err;
+    }
 
     if (s[pos][0] == '/') { /* remote is a unix socket */
 	connecter_str = alloc_sprintf("unix,%s", s[pos]);
     } else {
 	if (!validate_port(s[pos], s[pos + 1], &type, iaddr))
 	    goto out_err;
+	err = GE_INVAL;
 	connecter_str = alloc_sprintf("%s,%s,%s", type, s[pos], s[pos + 1]);
     }
+    if (!connecter_str) {
+	fprintf(stderr, "Unable to allocate connecter string");
+	err = GE_NOMEM;
+	goto out_err;
+    }
 
     if (remote)
 	err = add_remote_port(o, accepter_str, connecter_str, iaddr);
     else
-	err = add_local_port(o, accepter_str, connecter_str, iaddr);
+	err = add_local_port(locport, accepter_str, connecter_str, iaddr);
     goto out_err;
 
  out_not_enough_fields:
@@ -1514,6 +1537,8 @@ handle_port(struct gensio_os_funcs *o, bool remote, const char *iaddr)
     if (addr)
 	free(addr);
 
+    if (err)
+	err = -1;
     return err;
 }
 
@@ -1671,6 +1696,36 @@ lookup_mdns_transport(struct gensio_os_funcs *o, const char *name,
     return 0;
 }
 
+static bool
+strdup_double_backslash(char **str, gensiods *rlen)
+{
+    if (strchr(*str, '\\')) {
+	/* Double any \\ so they pass through properly. */
+	char *cp, *newstr;
+	size_t len = 0;
+
+	for (cp = *str; *cp; cp++) {
+	    len++;
+	    if (*cp == '\\')
+		len++;
+	}
+	newstr = malloc(len + 1);
+	if (!newstr)
+	    return true;
+	for (len = 0, cp = *str; *cp; cp++) {
+	    newstr[len++] = *cp;
+	    if (*cp == '\\')
+		newstr[len++] = *cp;
+	}
+	newstr[len] = '\0';
+	free(*str);
+	*str = newstr;
+	if (rlen)
+	    *rlen = len;
+    }
+    return false;
+}
+
 char *service;
 
 int
@@ -1710,8 +1765,6 @@ main(int argc, char *argv[])
     const char *mdns_type = NULL;
     const char *val_2fa = "", *pfx_2fa = "";
 
-    localport_err = pr_localport;
-
     memset(&userdata1, 0, sizeof(userdata1));
     memset(&userdata2, 0, sizeof(userdata2));
     userdata1.interactive = true;
@@ -1724,10 +1777,18 @@ main(int argc, char *argv[])
     }
     gensio_os_funcs_set_vlog(o, do_vlog);
 
+    locport = alloc_local_ports(o, pr_localport, NULL);
+    if (!locport) {
+	fprintf(stderr, "Could not allocate local port data\n");
+	gensio_os_funcs_free(o);
+	return 1;
+    }
+
     err = gensio_os_proc_setup(o, &proc_data);
     if (err) {
 	fprintf(stderr, "Could not setup process data: %s\n",
 		gensio_err_to_str(err));
+	free_local_ports(locport);
 	gensio_os_funcs_free(o);
 	return 1;
     }
@@ -1833,12 +1894,13 @@ main(int argc, char *argv[])
 	} else if ((err = cmparg(argc, argv, &arg, NULL, "--nointeractive",
 				 NULL))) {
 	    aux_data.flags |= GTLSSH_AUX_FLAG_NO_INTERACTIVE;
+	} else if ((err = cmparg(argc, argv, &arg, NULL, "--privileged",
+				 NULL))) {
+	    aux_data.flags |= GTLSSH_AUX_FLAG_PRIVILEGED;
 	} else if ((err = cmparg(argc, argv, &arg, "-L", NULL, &addr))) {
-	    if (!err)
-		err = handle_port(o, false, addr);
+	    err = handle_port(o, false, addr);
 	} else if ((err = cmparg(argc, argv, &arg, "-R", NULL, &addr))) {
-	    if (!err)
-		err = handle_port(o, true, addr);
+	    err = handle_port(o, true, addr);
 	} else if ((err = cmparg(argc, argv, &arg, "-4", NULL, NULL))) {
 	    iptype = "ipv4,";
 	} else if ((err = cmparg(argc, argv, &arg, "-6", NULL, NULL))) {
@@ -1899,7 +1961,7 @@ main(int argc, char *argv[])
 	}
 	hostname = s;
     } else {
-	username = get_my_username();
+	username = get_my_username(glogger, NULL);
 	if (!username)
 	    return 1;
 	hostname = argv[arg];
@@ -1938,7 +2000,7 @@ main(int argc, char *argv[])
 
 #ifdef _WIN32
 	if (!termvar)
-	    termvar = "ansi";
+	    termvar = "xterm";
 #endif
 	if (termvar) {
 	    len = 6 + 5 + strlen(termvar) + 2;
@@ -1963,12 +2025,12 @@ main(int argc, char *argv[])
     }
 
     if (!tlssh_dir) {
-	tlssh_dir = get_tlsshdir();
+	tlssh_dir = get_tlsshdir(glogger, NULL, NULL, NULL);
 	if (!tlssh_dir)
 	    return 1;
     }
 
-    err = checkout_file(tlssh_dir, true, true);
+    err = checkout_file(glogger, NULL, tlssh_dir, true, true);
     if (err)
 	return 1;
 
@@ -2040,6 +2102,12 @@ main(int argc, char *argv[])
 	}
     }
 
+    err = strdup_double_backslash(&username, NULL);
+    if (err) {
+	fprintf(stderr, "Out of memory allocating username\n");
+	return 1;
+    }
+
  retry:
     if (user_transport)
 	s = alloc_sprintf("%s%scertauth(enable-password,username=%s,%s,%s"
@@ -2153,7 +2221,7 @@ main(int argc, char *argv[])
     ioinfo_set_ready(ioinfo1, userdata1.io);
     ioinfo_set_ready(ioinfo2, userdata2.io);
 
-    start_local_ports(userdata2.io);
+    start_local_ports(locport, userdata2.io);
     start_remote_ports(ioinfo2);
 
     gensio_os_funcs_wait(o, userdata1.waiter, 1, NULL);
@@ -2203,6 +2271,7 @@ main(int argc, char *argv[])
     free_ser_ioinfo(subdata2);
 
     gensio_os_proc_cleanup(proc_data);
+    free_local_ports(locport);
     gensio_os_funcs_free(o);
     if (keyname)
 	free(keyname);
